# Copyright 2013 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

"""Methods for manipulating a time series.

A Kegbot core may report a time series (Drink.tick_time_series) for the meter
events that caused a drink.
"""

def from_string(s):
  """Converts a time series to a list of (int, int) tuples.

  The string should be a sequence of zero or more <time>:<amount> pairs.
  Whitespace delimits each pair; leading and trailing whitespace is ignored.

  ValueError is raised on any malformed input.
  """
  pairs = s.strip().split()
  ret = []
  for pair in pairs:
    time, amount = pair.split(':')
    time = int(time)
    amount = int(amount)
    if time < 0:
      raise ValueError('Time cannot be less than zero: %s' % time)
    ret.append((time, amount))
  return ret

def to_string(pairs):
  """Converts a series of (int, int) tuples to a time series string."""
  return ' '.join('%i:%i' % pair for pair in pairs)



# Copyright 2013 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

"""High-level interface to Django backend."""

from __future__ import absolute_import

import datetime
import logging

from django.conf import settings
from django.db import transaction
from django.utils import timezone
from pykeg.core import defaults
from pykeg.core import stats
from pykeg.core.cache import KegbotCache
from . import kb_common
from . import models
from . import time_series

from pykeg.web import tasks

class BackendError(Exception):
    """Base backend error exception."""

class NoTokenError(BackendError):
    """Token given is unknown."""

class KegbotBackend:
  """Provides high-level operations against the Kegbot system."""

  def __init__(self):
    self._logger = logging.getLogger('backend')
    self.cache = KegbotCache()

  @transaction.commit_on_success
  def CreateNewUser(self, username):
    """Creates and returns a User for the given username."""
    return models.User.objects.create(username=username)

  @transaction.commit_on_success
  def CreateTap(self, name, meter_name, relay_name=None, ml_per_tick=None):
    """Creates and returns a new KegTap.

    Args:
      name: The human-meaningful name for the tap, for instance, "Main Tap".
      meter_name: The unique sensor name for the tap, for instance,
          "kegboard.flow0".
      relay_name: If the tap is connected to a relay, this specifies its
          name, for instance, "kegboard.relay0".
      ml_per_tick: The number of milliliters per flow meter tick on this
          tap's meter.

    Returns:
        The new KegTap instance.
    """
    tap = models.KegTap.objects.create(name=name,
        meter_name=meter_name, relay_name=relay_name, ml_per_tick=ml_per_tick)
    tap.save()
    return tap

  @transaction.commit_on_success
  def CreateAuthToken(self, auth_device, token_value, username=None):
    """Creates a new AuthenticationToken.

    The combination of (auth_device, token_value) must be unique within the
    system.

    Args:
        auth_device: The name of the authentication device, for instance,
            "core.rfid".
        token_value: The opaque string value of the token, which is typically
            unique within the `auth_device` namespace.
        username: The User with which to associate this Token.

    Returns:
        The newly-created AuthenticationToken.
    """
    token = models.AuthenticationToken.objects.create(
        auth_device=auth_device, token_value=token_value)
    if username:
      user = get_user(username)
      token.user = user
    token.save()
    return token

  @transaction.commit_on_success
  def RecordDrink(self, tap_name, ticks, volume_ml=None, username=None,
      pour_time=None, duration=0, shout='', tick_time_series='',
      do_postprocess=True):
    """Records a new drink against a given tap.

    The tap must have a Keg assigned to it (KegTap.current_keg), and the keg
    must be active.

    Args:
        tap_name: The tap's meter_name.
        ticks: The number of ticks observed by the flow sensor for this drink.
        volume_ml: The volume, in milliliters, of the pour.  If specifed, this
            value is saved as the Drink's actual value.  If not specified, a
            volume is computed based on `ticks` and the KegTap's
            `ml_per_tick` setting.
        username: The username with which to associate this Drink, or None for
            an anonymous Drink.
        pour_time: The datetime of the drink.  If not supplied, the current
            date and time will be used.
        duration: Number of seconds it took to pour this Drink.  This is
            optional information not critical to the record.
        shout: A short comment left by the user about this Drink.  Optional.
        tick_time_series: Vector of flow update data, used for diagnostic
            purposes.
        do_postprocess: Set to False during bulk inserts.

    Returns:
        The newly-created Drink instance.
    """

    tap = self._GetTapFromName(tap_name)
    if not tap:
      raise BackendError("Tap unknown")
    if not tap.is_active or not tap.current_keg:
      raise BackendError("No active keg at this tap")

    if volume_ml is None:
      volume_ml = float(ticks) * tap.ml_per_tick

    user = None
    if username:
      user = get_user(username)
    else:
      user = models.SiteSettings.get().default_user

    if not pour_time:
      pour_time = timezone.now()

    keg = tap.current_keg

    if tick_time_series:
      try:
        # Validate the time series by parsing it; canonicalize it by generating
        # it again.  If malformed, just junk it; it's non-essential information.
        tick_time_series = time_series.to_string(time_series.from_string(tick_time_series))
      except ValueError, e:
        self._logger.warning('Time series invalid, ignoring. Error was: %s' % e)
        tick_time_series = ''

    d = models.Drink(ticks=ticks, keg=keg, user=user,
        volume_ml=volume_ml, time=pour_time, duration=duration,
        shout=shout, tick_time_series=tick_time_series)
    models.DrinkingSession.AssignSessionForDrink(d)
    d.save()

    keg.served_volume_ml += volume_ml
    keg.save(update_fields=['served_volume_ml'])

    self.cache.update_generation()

    if do_postprocess:
      stats.generate(d)
      events = models.SystemEvent.ProcessDrink(d)
      tasks.schedule_tasks(events)

    return d

  @transaction.commit_on_success
  def CancelDrink(self, drink, spilled=False):
    """Permanently deletes a Drink from the system.

    Associated data, such as SystemEvent, PourPicture, and other data, will
    be destroyed along with the drink. Statistics and session data will be
    recomputed after the drink is destroyed.

    Args:
        drink_id: The Drink, or id of the Drink, to cancel.
        spilled: If True, after canceling the Drink, the drink's volume will
            be added to its Keg's "spilled" total.  This is is typically useful
            for removing test pours from the system while still accounting for
            the lost volume.

    Returns:
        The deleted drink.
    """
    if not isinstance(drink, models.Drink):
      drink = models.Drink.objects.get(id=drink)

    session = drink.session
    drink_id = drink.id
    keg = drink.keg
    volume_ml = drink.volume_ml

    keg_update_fields = ['served_volume_ml']
    keg.served_volume_ml -= volume_ml

    # Transfer volume to spillage if requested.
    if spilled and volume_ml and drink.keg:
      keg.spilled_ml += volume_ml
      keg_update_fields.append('spilled_ml')

    keg.save(update_fields=keg_update_fields)

    # Invalidate all statistics.
    stats.invalidate(drink)

    # Delete the drink, including any objects related to it.
    drink.delete()

    session.Rebuild()

    # Regenerate stats for any drinks following the just-deleted one.
    for drink in models.Drink.objects.filter(id__gt=drink_id).order_by('id'):
      stats.generate(drink)

    self.cache.update_generation()

    return drink

  @transaction.commit_on_success
  def AssignDrink(self, drink, user):
    """Assigns, or re-assigns, a previously-recorded Drink.

    Statistics and session data will be recomputed as a side-effect
    of any change to user assignment.  (If the drink is already assigned
    to the given user, this method is a no-op).

    Args:
        drink: The Drink object (or drink id) to assign.
        user: The User object (or username) to set as the owner,
            or None for anonymous.

    Returns:
        The drink.
    """
    drink = get_drink(drink)
    user = get_user(user)
    if drink.user == user:
      return drink

    drink.user = user
    drink.save()

    stats.invalidate(drink)

    for e in drink.events.all():
      e.user = user
      e.save()
    for p in drink.pictures.all():
      p.user = user
      p.save()

    drink.session.Rebuild()
    stats.generate(drink)

    self.cache.update_generation()
    return drink

  @transaction.commit_on_success
  def SetDrinkVolume(self, drink, volume_ml):
    """Updates the drink volume."""
    if volume_ml == drink.volume_ml:
      return

    difference = volume_ml - drink.volume_ml
    drink.volume_ml = volume_ml
    drink.save(update_fields=['volume_ml'])

    keg = drink.keg
    keg.served_volume_ml += difference
    keg.save(update_fields=['served_volume_ml'])

    stats.invalidate(drink)
    drink.session.Rebuild()

    # Regenerate stats for this drink and all subsequent.
    for drink in models.Drink.objects.filter(id__gte=drink.id).order_by('id'):
      stats.generate(drink)

    self.cache.update_generation()

  @transaction.commit_on_success
  def LogSensorReading(self, sensor_name, temperature, when=None):
    """Logs a ThermoSensor reading.

    To avoid an excessive number of entries, the system limits temperature
    readings to one per minute.  If there is already a recording for the
    given time period, that record will be updated with the current temperature
    ("last one wins").

    Regardless of this record's timestamp, any records older than
    `kb_common.THERMO_SENSOR_HISTORY_MINUTES` will be deleted as a side effect
    of this call.

    Args:
        sensor_name: The name of the sensor, corresponding to
            ThermoSensor.raw_name.
        temperature: Temperature, in celsius degrees.  Values outside of the
            range specified in `kb_common.THERMO_SENSOR_RANGE` will be
            rejected.
        when: If specified, a datetime of the recording, otherwise the current
            time is used.

    Returns:
        The record for this reading.
    """
    now = timezone.now()
    if not when:
      when = now

    # The maximum resolution of ThermoSensor records is 1 minute.  Round the
    # time down to the nearest minute; if a record already exists for this time,
    # replace it.
    when = when.replace(second=0, microsecond=0)

    # If the temperature is out of bounds, reject it.
    min_val = kb_common.THERMO_SENSOR_RANGE[0]
    max_val = kb_common.THERMO_SENSOR_RANGE[1]
    if temperature < min_val or temperature > max_val:
      raise ValueError('Temperature out of bounds')

    sensor = self._GetSensorFromName(sensor_name)
    defaults = {
        'temp': temperature,
    }
    record, created = models.Thermolog.objects.get_or_create(sensor=sensor,
      time=when, defaults=defaults)
    record.temp = temperature
    record.save()

    # Delete old entries.
    keep_time = now - datetime.timedelta(
        minutes=kb_common.THERMO_SENSOR_HISTORY_MINUTES)
    old_entries = models.Thermolog.objects.filter(time__lt=keep_time)
    old_entries.delete()

    return record

  @transaction.commit_on_success
  def GetAuthToken(self, auth_device, token_value):
    """Fetches the AuthenticationToken matching the given parameters.

    Args:
        auth_device: The requested token's auth_device.
        token_value: The token's value, typically unique within auth_device.

    Returns:
        The matching token.

    Raises:
        NoTokenError: No matching token could be found.
    """
    if token_value and auth_device in kb_common.AUTH_MODULE_NAMES_HEX_VALUES:
      token_value = token_value.lower()
    try:
      return models.AuthenticationToken.objects.get(auth_device=auth_device,
          token_value=token_value)
    except models.AuthenticationToken.DoesNotExist:
      # TODO(mikey): return None instead of raising.
      raise NoTokenError

  @transaction.commit_on_success
  def StartKeg(self, tap, beer_type=None, keg_size=None,
          brewer_name=None, beer_name=None, style_name=None):
      """Activates a new keg at the given tap.

      The tap must be inactive (tap.current_keg == None), otherwise a
      ValueError will be thrown.

      A beer type must be specified, either by providing an existing
      BeerType instance as `beer_type`, or by specifying values for
      `brewer_name`, `beer_name`, and `style_name`.

      When using the latter form, the system will attempt to match
      the string type parameters against an already-existing BeerType.
      Otherwise, a new BeerType will be created.

      Args:
          tap: The KegTap object to tap against, or a string matching
              KegTap.meter_name.
          beer_type: The type of beer, as a beer_type object.  If specified,
              brewer_name, beer_name, and style_name must all be None.
          keg_size: The size of this keg.  If unspecified, the default
              size will be used.
          brewer_name: The Brewer's name for the keg's beer.  Must be
              given with beer_name and style_name; beer_type must be None.
          beer_name: The BeerType's name for the keg's beer.  Must be
              given with brewer_name and style_name; beer_type must be
              None.
          style_name: The BeerStyle for the keg's beer.  Must be given
              with brewer_name and beer_name; beer_type must be None.

      Returns:
          The new keg instance.
      """
      if not isinstance(tap, models.KegTap):
          tap = models.KegTap.objects.get(meter_name=tap)

      if tap.is_active():
          raise ValueError('Tap is already active, must end keg first.')

      if beer_type:
          if brewer_name or beer_name or style_name:
              raise ValueError(
                  'Cannot give brewer_name, beer_name, or style_name with beer_type')
      else:
          if not beer_name:
              raise ValueError('Must supply beer_name when beer_type is None')
          if not brewer_name:
              raise ValueError('Must supply brewer_name when beer_type is None')
          if not style_name:
              raise ValueError('Must supply style_name when beer_type is None')
          beer_style = models.BeerStyle.objects.get_or_create(name=style_name)[0]
          brewer = models.Brewer.objects.get_or_create(name=brewer_name)[0]
          beer_type = models.BeerType.objects.get_or_create(name=beer_name,
                  brewer=brewer, style=beer_style)[0]

      if not keg_size:
          keg_size = defaults.get_default_keg_size()

      keg = models.Keg.objects.create(type=beer_type, size=keg_size,
              online=True, full_volume_ml=keg_size.volume_ml)

      old_keg = tap.current_keg
      if old_keg:
          old_keg.online = False
          old_keg.save()

      tap.current_keg = keg
      tap.save()

      return keg

  @transaction.commit_on_success
  def EndKeg(self, tap):
      """Takes the current Keg offline at the given tap.

      Args:
          tap: The KegTap object to tap against, or a string matching
              KegTap.meter_name.

      Returns:
          The old keg.

      Raises:
          ValueError: if the tap is missing or already offline.
      """
      if not isinstance(tap, models.KegTap):
          tap = models.KegTap.objects.get(meter_name=tap)

      if not tap.current_keg:
          raise ValueError('Tap is already offline.')

      keg = tap.current_keg
      keg.online = False
      keg.save()

      tap.current_keg = None
      tap.save()

      return keg

  def _GetTapFromName(self, tap_name):
    """"Returns a KegTap object with meter_name matching tap_name, or None."""
    try:
      return models.KegTap.objects.get(meter_name=tap_name)
    except models.KegTap.DoesNotExist:
      return None

  def _GetKegForTapName(self, tap_name):
    """Returns the current keg at tap matching tap_name, or None."""
    tap = self._GetTapFromName(tap_name)
    if tap and tap.current_keg and tap.current_keg.online:
      return tap.current_keg
    return None

  def _GetSensorFromName(self, name, autocreate=True):
    """Returns the TemperatureSensor with raw_name matching name.

    Args:
        name: The sensor's raw_name.
        autocreate: If True, create a new sensor if one does not exist.

    Returns:
        The TemperatureSensor object, or None if none exists and autocreate was
        not True.
    """
    try:
      return models.ThermoSensor.objects.get(raw_name=name)
    except models.ThermoSensor.DoesNotExist:
      if autocreate:
        sensor = models.ThermoSensor(raw_name=name, nice_name=name)
        sensor.save()
        return sensor
      else:
        return None

def get_user(user_or_username):
  """Returns the User object for the given username, or None."""
  if not user_or_username:
    return None
  if not isinstance(user_or_username, models.User):
    return models.User.objects.get(username=user_or_username)
  return user_or_username

def get_drink(drink_or_id):
  if not isinstance(drink_or_id, models.Drink):
    return models.Drink.objects.get(pk=drink_or_id)
  return drink_or_id



# Copyright 2013 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

"""Plugin interface for extending the Kegbot frontend."""

import logging
from pykeg.plugin import models
from kegbot.util.util import AttrDict

class PluginDatastore:
    """Interface for plugins to persist plugin-specific data."""

    def __init__(self, plugin_name):
        self.plugin_name = plugin_name

    def set(self, key, value):
        """Store plugin-specific data."""
        if value is None:
            self.delete(key)
            return
        try:
            row = models.PluginData.objects.get(plugin_name=self.plugin_name, key=key)
            row.value = value
            row.save()
        except models.PluginData.DoesNotExist:
            models.PluginData.objects.create(plugin_name=self.plugin_name, key=key,
                value=value)

    def get(self, key, default=None):
        """Fetch plugin-specific data."""
        try:
            row = models.PluginData.objects.get(plugin_name=self.plugin_name,
                key=key)
            return row.value
        except models.PluginData.DoesNotExist:
            return default

    def delete(self, key):
        try:
            models.PluginData.objects.get(plugin_name=self.plugin_name,
                key=key).delete()
        except models.PluginData.DoesNotExist:
            pass

class Plugin:
    """Interface class for plugins."""

    NAME = None
    SHORT_NAME = None
    DESCRIPTION = None
    VERSION = None
    URL = None

    def __init__(self, datastore):
        self.datastore = datastore
        self.logger = logging.getLogger(self.SHORT_NAME)

    @classmethod
    def get_version(cls):
        """Return the curent plugin version.

        Subclasses should set the class attribute VERSION, which is
        returned by the default implementation.

        Returns:
            The version, as a tuple of (major, minor, micro[, tag]).
            Major, minor, and micro must be integers; tag, if present,
            must be an arbitrary string.
        """
        if not cls.VERSION:
            raise NotImplementedError
        return cls.VERSION

    @classmethod
    def get_name(cls):
        """Return a human-readable (one or two word) name for this plugin.

        Subclasses should set the class attribute NAME, which is
        returned by the default implementation.
        """
        if not cls.NAME:
            raise NotImplementedError
        return cls.NAME

    @classmethod
    def get_short_name(cls):
        """Return a short for this plugin.

        Subclasses should set the class attribute NAME, which is
        returned by the default implementation.
        """
        if not cls.SHORT_NAME:
            raise NotImplementedError
        return cls.SHORT_NAME

    @classmethod
    def get_description(cls):
        """Return a concise human-readable description for this plugin.

        Subclasses should set the class attribute DESCRIPTION, which is
        returned by the default implementation.
        """
        if not cls.DESCRIPTION:
            raise NotImplementedError
        return cls.DESCRIPTION

    @classmethod
    def get_url(cls):
        """Returns the information URL (homepage) for the plugin.

        Subclassses should set the class attribute URL, which is returned
        by the default implementation.
        """
        if not cls.URL:
            raise NotImplementedError
        return cls.URL

    ### Plugin methods

    def get_admin_settings_view(self):
        """Returns the view instance for the main admin settings for this
        plugin, or None.
        """
        return None

    def get_extra_admin_views(self):
        """Returns an iterable of additional views to be installed in the admin
        site section this plugin.

        Each item should be a 3-tuple of the form:
          (regex, view name, url name)

        Each view will be installed with the name
        "plugin-<plugin name>-<url name>"
        """
        return []

    def get_user_settings_view(self):
        """Returns the view instance for the main user settings for this
        plugin, or None.
        """
        return None

    def get_extra_user_views(self):
        """Returns an iterable of additional views to be installed in the
        user section for this plugin.

        Each item should be a 3-tuple of the form:
          (regex, view name, url name)

        Each view will be installed with the name
        "plugin-<plugin name>-<url name>"
        """
        return []

    def handle_new_event(self, event):
        """Called synchronously when a new event is posted.

        Plugins should *quickly* perform any work. Long-running work can be
        performed by scheduling a background task in this method."""
        pass

    ### Helpers

    def save_form(self, form, prefix):
        """Helper method to save a form using the specified per-field prefix."""
        for field_name, value in form.cleaned_data.iteritems():
            self.datastore.set('%s:%s' % (prefix, field_name), value)

    def get_saved_form_data(self, form, prefix):
        """Helper method to load a form using the specified per-field prefix."""
        ret = {}
        for field_name, field in form.fields.iteritems():
            initial = self.datastore.get('%s:%s' % (prefix, field_name))
            if initial is not None:
                ret[field_name] = field.to_python(initial)
        return AttrDict(ret)

    def load_form_defaults(self, form, prefix):
        """Applies defaults to a form using `get_saved_form_data`."""
        defaults = self.get_saved_form_data(form, prefix)
        for field_name, value in defaults.iteritems():
            form.fields[field_name].initial = value
        return form




from django.conf import settings

"""Methods to query configuration of optional features."""

def use_facebook():
  if not getattr(settings, 'FACEBOOK_API_KEY', ''):
    return False
  if not getattr(settings, 'FACEBOOK_SECRET_KEY', ''):
    return False
  return True



#!/usr/bin/env python
#
# Copyright 2009 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

import os
import sys
from pykeg.core.kb_common import LOCAL_SETTINGS_SEARCH_DIRS

_DEBUG = False

_CUSTOM_SEARCH_DIR = os.getenv('KEGBOT_SETTINGS_DIR')
if _CUSTOM_SEARCH_DIR:
  SEARCH_DIRS = [os.path.expanduser(_CUSTOM_SEARCH_DIR)]
else:
  SEARCH_DIRS = [os.path.expanduser(d) for d in LOCAL_SETTINGS_SEARCH_DIRS]

def _Debug(message):
  if _DEBUG:
    sys.stderr.write('importhacks: %s\n' % (message,))

def _Warning(message):
  if _DEBUG:
    sys.stderr.write('importhacks: %s\n' % (message,))

def _AddToSysPath(paths):
  for path in paths:
    path = os.path.abspath(path)
    if path not in sys.path:
      _Debug('Adding to sys.path: %s' % path)
      sys.path.append(path)
    else:
      _Debug('Already in sys.path: %s' % path)

def _ExtendSysPath():
  """ Add some paths where we'll look for user settings. """
  paths = []
  for dir in SEARCH_DIRS:
    paths.append(dir)
    if _DEBUG:
      test_settings = os.path.join(dir, 'local_settings.py')
      if os.path.exists(test_settings):
        _Debug('%s exists' % test_settings)
      else:
        _Debug('%s does NOT exist' % test_settings)

  _AddToSysPath(paths)

def _SetDjangoSettingsEnv(settings='pykeg.settings'):
  """ Set django settings if not set. """
  if not os.environ.get('DJANGO_SETTINGS_MODULE'):
    _Debug('Setting DJANGO_SETTINGS_MODULE=%s' % (settings,))
    os.environ['DJANGO_SETTINGS_MODULE'] = settings

def _FixAll():
  try:
    import pykeg
    pykeg_dir = os.path.dirname(pykeg.__file__)
    _Debug('Pykeg loaded from dir: %s' % pykeg_dir)
  except ImportError:
    _Warning('Error: pykeg could not be imported')
    sys.exit(1)

  _ExtendSysPath()
  _SetDjangoSettingsEnv()

  try:
    import local_settings
    common_dir = os.path.dirname(local_settings.__file__)
    _Debug('local_settings loaded from dir: %s' % common_dir)
  except ImportError:
    _Warning('Warning: local_settings could not be imported')

if __name__ == '__main__' or os.environ.get('DEBUG_IMPORT_HACKS'):
  # When run as a program, or DEBUG_IMPORT_HACKS is set: print debug info
  _DEBUG = True

_FixAll()



from django import forms
from crispy_forms.helper import FormHelper
from crispy_forms.layout import Layout, Div, Submit, HTML, Button, Row, Field, Hidden, Div
from crispy_forms.bootstrap import AppendedText, PrependedText, FormActions

from kegbot.util import units
from pykeg.core import backend
from pykeg.core import models

ALL_TAPS = models.KegTap.objects.all()
ALL_SIZES = models.KegSize.objects.all()
ALL_KEGS = models.Keg.objects.all()

class GeneralSettingsForm(forms.Form):
  name = forms.CharField(help_text='Name of this Kegbot system')

class ChangeKegForm(forms.Form):
  keg_size = forms.ModelChoiceField(queryset=ALL_SIZES,
      required=True,
      empty_label=None)

  beer_name = forms.CharField(required=True, label='Beer Name')
  beer_id = forms.CharField(widget=forms.HiddenInput(), required=False)
  brewer_name = forms.CharField(required=True, label='Brewer')
  brewer_id = forms.CharField(widget=forms.HiddenInput(), required=False)
  style_name = forms.CharField(required=True, label='Style',
    help_text='Example: Pale Ale, Stout, etc.')
  style_id = forms.CharField(widget=forms.HiddenInput(), required=False)

  description = forms.CharField(required=False,
      help_text='Public description of this specific keg (optional)')

  helper = FormHelper()
  helper.form_class = 'form-horizontal beer-select'
  helper.layout = Layout(
      Field('beer_name', css_class='input-xlarge'),
      Field('beer_id', type='hidden'),
      Field('brewer_name', css_class='input-xlarge'),
      Field('brewer_id', type='hidden'),
      Field('style_name', css_class='input-xlarge'),
      Field('style_id', type='hidden'),
      Field('keg_size', css_class='input-xlarge'),
      Field('description', css_class='input-xlarge'),
      FormActions(
          Submit('submit_change_keg_form', 'Activate Keg', css_class='btn-primary'),
      )
  )

  def save(self, tap):
    if not self.is_valid():
      raise ValueError('Form is not valid.')
    b = backend.KegbotBackend()

    if tap.is_active():
      b.EndKeg(tap)

    cd = self.cleaned_data
    keg = b.StartKeg(tap, beer_name=cd['beer_name'], brewer_name=cd['brewer_name'],
        style_name=cd['style_name'], keg_size=cd['keg_size'])

    if cd['description']:
      keg.description = cd['description']
      keg.save()


class EndKegForm(forms.Form):
  keg = forms.ModelChoiceField(queryset=ALL_KEGS, widget=forms.HiddenInput)

  helper = FormHelper()
  helper.form_class = 'form-horizontal beer-select'
  helper.layout = Layout(
      Field('keg', type='hidden'),
      FormActions(
        Submit('submit_end_keg_form', 'End Keg', css_class='btn-danger'),
      )
  )


class TapForm(forms.ModelForm):
  class Meta:
    model = models.KegTap
    fields = ('name', 'meter_name', 'relay_name', 'description',
        'temperature_sensor', 'ml_per_tick')

  def __init__(self, *args, **kwargs):
    site = kwargs.pop('site', None)
    super(TapForm, self).__init__(*args, **kwargs)
    self.fields['temperature_sensor'].queryset = models.ThermoSensor.objects.all()
    self.fields['temperature_sensor'].empty_label = 'No sensor.'

  helper = FormHelper()
  helper.form_class = 'form-horizontal'
  helper.layout = Layout(
      Field('name', css_class='input-xlarge'),
      Field('meter_name', css_class='input-xlarge'),
      Field('relay_name', css_class='input-xlarge'),
      Field('temperature_sensor', css_class='input-xlarge'),
      Field('ml_per_tick', css_class='input-xlarge'),
      Field('description', css_class='input-xlarge'),
      FormActions(
          Submit('submit_tap_form', 'Save Settings', css_class='btn-success'),
      )
  )


class DeleteTapForm(forms.Form):
  helper = FormHelper()
  helper.form_class = 'form-horizontal'
  helper.layout = Layout(
      FormActions(
          Submit('submit_delete_tap_form', 'Delete Tap', css_class='btn-danger'),
      )
  )


class SiteSettingsForm(forms.ModelForm):
  class Meta:
    model = models.SiteSettings
    fields = (
        'title',
        'description',
        'privacy',
        'volume_display_units',
        'temperature_display_units',
        'timezone',
        'hostname',
        'use_ssl',
        'web_hook_urls',
        'session_timeout_minutes',
        'google_analytics_id',
        'guest_name',
        'guest_image',
        'default_user',
        'registration_allowed',
        'registration_confirmation',
        'allowed_hosts',
    )

  helper = FormHelper()
  helper.form_class = 'form-horizontal'
  helper.layout = Layout(
      Field('title', css_class='input-xlarge'),
      Field('description', css_class='input-xlarge'),
      Field('privacy', css_class='input-xlarge'),
      Field('volume_display_units', css_class='input-xlarge'),
      Field('temperature_display_units', css_class='input-xlarge'),
      Field('timezone'),
      Field('hostname'),
      Field('use_ssl'),
      Field('web_hook_urls'),
      Field('session_timeout_minutes'),
      Field('google_analytics_id'),
      Field('guest_name'),
      Field('guest_image'),
      Field('default_user'),
      Field('registration_allowed', css_class='input-xlarge'),
      Field('registration_confirmation', css_class='input-xlarge'),
      Field('allowed_hosts'),
      FormActions(
          Submit('submit', 'Save Settings', css_class='btn-primary'),
      )
  )

class BeerTypeForm(forms.ModelForm):
  class Meta:
    model = models.BeerType
    fields = ('name', 'style', 'brewer', 'edition', 'abv',
        'calories_oz', 'carbs_oz', 'original_gravity', 'specific_gravity',
        'untappd_beer_id')

  new_image = forms.ImageField(required=False,
      help_text='Set/replace image for this beer type.')

  helper = FormHelper()
  helper.form_class = 'form-horizontal'
  helper.layout = Layout(
      Field('name', css_class='input-xlarge'),
      Field('style', css_class='input-xlarge'),
      Field('brewer'),
      Field('edition'),
      Field('abv'),
      Field('calories_oz'),
      Field('carbs_oz'),
      Field('original_gravity'),
      Field('specific_gravity'),
      Field('untappd_beer_id'),
      Field('new_image'),
      FormActions(
          Submit('submit', 'Save', css_class='btn-primary'),
      )
  )

class BrewerForm(forms.ModelForm):
  class Meta:
    model = models.Brewer

  helper = FormHelper()
  helper.form_class = 'form-horizontal'
  helper.layout = Layout(
      Field('name', css_class='input-xlarge'),
      Field('country', css_class='input-xlarge'),
      Field('origin_state', css_class='input-xlarge'),
      Field('origin_city', css_class='input-xlarge'),
      Field('production'),
      Field('url'),
      Field('description'),
      Field('image'),
      FormActions(
          Submit('submit', 'Save', css_class='btn-primary'),
      )
  )

class BeerStyleForm(forms.ModelForm):
  class Meta:
    model = models.BeerStyle

  helper = FormHelper()
  helper.form_class = 'form-horizontal'
  helper.layout = Layout(
      Field('name', css_class='input-xlarge'),
      FormActions(
          Submit('submit', 'Save', css_class='btn-primary'),
      )
  )

class FindUserForm(forms.Form):
  username = forms.CharField()

class UserForm(forms.ModelForm):
  class Meta:
    model = models.User
    fields = (
        'username',
        'email',
        'password',
        'is_staff',
    )

  helper = FormHelper()
  helper.form_class = 'form-horizontal'
  helper.layout = Layout(
      Field('username', css_class='input-xlarge'),
      Field('email', css_class='input-xlarge'),
      'password',
      'is_staff',
      FormActions(
          Submit('submit', 'Save', css_class='btn-primary'),
      )
  )

class TokenForm(forms.ModelForm):
  class Meta:
    model = models.AuthenticationToken
    fields = (
        'nice_name',
        'enabled',
    )
  username = forms.CharField(required=False)

  helper = FormHelper()
  helper.form_class = 'form-horizontal user-select'
  helper.layout = Layout(
      Field('username', css_class='input-xlarge'),
      Field('nice_name', css_class='input-xlarge'),
      'enabled',
      FormActions(
          Submit('submit', 'Save', css_class='btn-primary'),
      )
  )

  def clean_username(self):
    username = self.cleaned_data['username']
    if username == '':
      self.cleaned_data['user'] = None
      return
    try:
      self.cleaned_data['user'] = models.User.objects.get(username=username)
    except models.User.DoesNotExist:
      raise forms.ValidationError('Invalid username; use a complete user name or leave blank.')
    return username


class AddTokenForm(forms.ModelForm):
  class Meta:
    model = models.AuthenticationToken
    fields = (
        'auth_device',
        'token_value',
        'enabled',
    )
  CHOICES = (
    ('core.rfid', 'RFID'),
    ('core.onewire', 'OneWire/iButton'),
    ('nfc', 'NFC'),
  )
  auth_device = forms.ChoiceField(choices=CHOICES)
  username = forms.CharField(required=False)

  helper = FormHelper()
  helper.form_class = 'form-horizontal user-select'
  helper.layout = Layout(
      Field('auth_device', css_class='input-xlarge'),
      Field('token_value', css_class='input-xlarge'),
      Field('username', css_class='input-xlarge'),
      'enabled',
      FormActions(
          Submit('submit', 'Save', css_class='btn-primary'),
      )
  )

  def clean_username(self):
    username = self.cleaned_data['username']
    if username == '':
      self.cleaned_data['user'] = None
      return
    try:
      self.cleaned_data['user'] = models.User.objects.get(username=username)
    except models.User.DoesNotExist:
      raise forms.ValidationError('Invalid username; use a complete user name or leave blank.')
    return username


class CancelDrinkForm(forms.Form):
  pass

class ReassignDrinkForm(forms.Form):
  username = forms.CharField(required=False)

class ChangeDrinkVolumeForm(forms.Form):
  UNIT_CHOICES = (
    ('mL', 'mL'),
    ('oz', 'oz')
  )
  units = forms.ChoiceField(required=True, choices=UNIT_CHOICES)
  volume = forms.FloatField(required=True, min_value=0)

  def clean_volume(self):
    volume = self.cleaned_data['volume']
    if self.cleaned_data['units'] == 'oz':
      self.cleaned_data['volume_ml'] = float(units.Quantity(volume, units.UNITS.Ounce).InMilliliters())
    else:
      self.cleaned_data['volume_ml'] = volume
    return volume

class RecordDrinkForm(forms.Form):
  units = forms.ChoiceField(required=True, choices=ChangeDrinkVolumeForm.UNIT_CHOICES)
  volume = forms.FloatField(required=True, min_value=0)
  username = forms.CharField(required=False)

  def clean_username(self):
    username = self.cleaned_data['username']
    if username == '':
      self.cleaned_data['user'] = None
      return
    try:
      self.cleaned_data['user'] = models.User.objects.get(username=username)
    except models.User.DoesNotExist:
      raise forms.ValidationError('Invalid username; use a complete user name or leave blank.')
    return username

  def clean_volume(self):
    volume = self.cleaned_data['volume']
    if self.cleaned_data['units'] == 'oz':
      self.cleaned_data['volume_ml'] = float(units.Quantity(volume, units.UNITS.Ounce).InMilliliters())
    else:
      self.cleaned_data['volume_ml'] = volume
    return volume



# Copyright 2013 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

"""General tests for the web interface."""

from django.test import TransactionTestCase
from pykeg.core import backend
from pykeg.core import models
from pykeg.core import defaults

class KegwebTestCase(TransactionTestCase):
    def setUp(self):
        defaults.set_defaults(set_is_setup=True)

    def testBasicEndpoints(self):
        for endpoint in ('/kegs/', '/stats/'):
            response = self.client.get(endpoint)
            self.assertEquals(200, response.status_code)

        for endpoint in ('/sessions/',):
            response = self.client.get(endpoint)
            self.assertEquals(404, response.status_code)

        b = backend.KegbotBackend()
        keg = b.StartKeg('kegboard.flow0', beer_name='Unknown', brewer_name='Unknown',
            style_name='Unknown')
        self.assertIsNotNone(keg)
        response = self.client.get('/kegs/')
        self.assertEquals(200, response.status_code)

        d = b.RecordDrink('kegboard.flow0', ticks=100)
        drink_id = d.id

        response = self.client.get('/d/%s' % drink_id, follow=True)
        self.assertRedirects(response, '/drinks/%s' % drink_id, status_code=301)

        session_id = d.session.id
        response = self.client.get('/s/%s' % session_id, follow=True)
        self.assertRedirects(response, d.session.get_absolute_url(), status_code=301)

    def testShout(self):
        b = backend.KegbotBackend()
        keg = b.StartKeg('kegboard.flow0', beer_name='Unknown', brewer_name='Unknown',
            style_name='Unknown')
        d = b.RecordDrink('kegboard.flow0', ticks=123, shout='_UNITTEST_')
        response = self.client.get(d.get_absolute_url())
        self.assertContains(response, '<p>_UNITTEST_</p>', status_code=200)






# Copyright 2012 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

from pykeg import EPOCH

from pykeg.core import models
from pykeg.core.backend import KegbotBackend
from pykeg.core.cache import KegbotCache
from pykeg.web.api.util import is_api_request

from pykeg.plugin import util as plugin_util

from django.db import DatabaseError
from django.conf import settings
from django.contrib import messages
from django.core.urlresolvers import reverse
from django.http import HttpResponse
from django.http import Http404
from django.http import HttpResponseRedirect
from django.shortcuts import get_object_or_404
from django.template.response import SimpleTemplateResponse
from django.template import RequestContext
from django.utils import timezone

# TODO(mikey): rename me
ALLOWED_PATHS = (
    '/api/login/',
    '/api/get-api-key/',
    '/accounts/login/',
    '/admin/',
    '/media/',
    '/setup/',
)

def _path_allowed(path, kbsite):
  for p in ALLOWED_PATHS:
    if path.startswith(p):
      return True
  return False


class KegbotSiteMiddleware:
  ALLOWED_VIEW_MODULE_PREFIXES = (
      'pykeg.web.setup_wizard.',
  )
  def process_request(self, request):
    epoch = None
    request.need_setup = False
    request.need_upgrade = False

    try:
      request.kbsite = models.KegbotSite.objects.get(name='default')
      epoch = request.kbsite.epoch
    except (models.KegbotSite.DoesNotExist, DatabaseError), e:
      request.kbsite = None

    if not request.kbsite or not request.kbsite.is_setup:
      request.need_setup = True
    else:
      if not epoch or epoch < EPOCH:
        request.need_upgrade = True
      timezone.activate(request.kbsite.settings.timezone)
      request.plugins = dict((p.get_short_name(), p) for p in plugin_util.get_plugins())

    request.kbcache = KegbotCache()
    request.backend = KegbotBackend()
    return None

  def process_view(self, request, view_func, view_args, view_kwargs):
    for prefix in self.ALLOWED_VIEW_MODULE_PREFIXES:
      if view_func.__module__.startswith(prefix):
        return None

    if is_api_request(request):
      # API endpoints handle "setup required" differently.
      return None

    if request.need_setup:
      return self._setup_required(request)
    elif request.need_upgrade:
      return self._upgrade_required(request)

    return None

  def process_response(self, request, response):
    if request.method in ('POST', 'PUT', 'PATCH') and response.status_code < 400:
      # Invalidate cache on any successful change.
      # TODO(mikey): More granular cache invalidations.
      request.kbcache.update_generation()
    return response

  def _setup_required(self, request):
    return SimpleTemplateResponse('setup_wizard/setup_required.html',
        context=RequestContext(request), status=403)

  def _upgrade_required(self, request, current_epoch=None):
    context = RequestContext(request)
    context['current_epoch'] = current_epoch
    return SimpleTemplateResponse('setup_wizard/upgrade_required.html',
        context=context, status=403)


class SiteActiveMiddleware:
  """Middleware which throws 503s when KegbotSite.is_active is false."""
  def process_view(self, request, view_func, view_args, view_kwargs):
    if not hasattr(request, 'kbsite') or not request.kbsite:
      return None
    kbsite = request.kbsite

    # We have a KegbotSite, and that site is active: nothing to do.
    if kbsite.is_active:
      return None

    # If the request is for a whitelisted path, allow it.
    if _path_allowed(request.path, kbsite):
      return None

    # Allow staff/superusers access if inactive.
    if request.user.is_staff or request.user.is_superuser:
      return None

    return HttpResponse('Site temporarily unavailable', status=503)

class HttpHostMiddleware:
  """Middleware which checks a dynamic version of settings.ALLOWED_HOSTS."""
  def process_request(self, request):
    if not getattr(request, 'kbsite', None):
      return None

    host = request.get_host()
    allowed_hosts_str = request.kbsite.settings.allowed_hosts
    if allowed_hosts_str:
      host_patterns = allowed_hosts_str.strip().split()
    else:
      host_patterns = []
    valid = HttpHostMiddleware.validate_host(host, host_patterns)

    if not valid:
      message =  "Invalid HTTP_HOST header (you may need to change Kegbot's ALLOWED_HOSTS setting): %s" % host
      if request.user.is_superuser or request.user.is_staff:
        messages.warning(request, message)
      else:
        raise SuspiciousOperation(message)

  @classmethod
  def validate_host(cls, host, allowed_hosts):
    """Clone of django.http.request.validate_host.

    Local differences: treats an empty `allowed_hosts` list as a pass.
    """
    # Validate only the domain part.
    if not allowed_hosts:
      return True

    if host[-1] == ']':
        # It's an IPv6 address without a port.
        domain = host
    else:
        domain = host.rsplit(':', 1)[0]

    for pattern in allowed_hosts:
      pattern = pattern.lower()
      match = (
          pattern == '*' or
          pattern.startswith('.') and (
              domain.endswith(pattern) or domain == pattern[1:]
              ) or
          pattern == domain
          )
      if match:
          return True

    return False


class PrivacyMiddleware:
  """Enforces site privacy settings.

  Must be installed after ApiRequestMiddleware (in request order) to
  access is_kb_api_request attribute.
  """
  def process_view(self, request, view_func, view_args, view_kwargs):
    if not hasattr(request, 'kbsite'):
      return None
    elif _path_allowed(request.path, request.kbsite):
      return None
    elif request.is_kb_api_request:
      # api.middleware will enforce access requirements.
      return None

    privacy = request.kbsite.settings.privacy

    if privacy == 'public':
      return None
    elif privacy == 'staff':
      if not request.user.is_staff:
        return SimpleTemplateResponse('kegweb/staff_only.html',
            context=RequestContext(request), status=401)
      return None
    elif privacy == 'members':
      if not request.user.is_authenticated or not request.user.is_active:
        return SimpleTemplateResponse('kegweb/members_only.html',
            context=RequestContext(request), status=401)
      return None

    return HttpResponse('Server misconfigured, unknown privacy setting:%s' % privacy, status=500)




# Copyright 2013 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

"""Twitter plugin forms."""

from django import forms

WIDE_TEXT = forms.TextInput(attrs={'class': 'input-block-level'})

class CredentialsForm(forms.Form):
    consumer_key = forms.CharField(required=False)
    consumer_secret = forms.CharField(required=False)

class SiteSettingsForm(forms.Form):
    tweet_keg_events = forms.BooleanField(initial=True, required=False,
        help_text='Tweet when a keg is started or ended.')
    tweet_session_events = forms.BooleanField(initial=True, required=False,
        help_text='Tweet when a new session is started.')
    tweet_drink_events = forms.BooleanField(initial=False, required=False,
        help_text='Tweet whenever a drink is poured (caution: potentially annoying).')
    include_guests = forms.BooleanField(initial=True, required=False,
        help_text='When tweeting drink events, whether to include guest pours.')
    append_url = forms.BooleanField(initial=True, required=False,
        help_text='Whether to append drink URLs to tweets.')
    session_started_template = forms.CharField(max_length=255, widget=WIDE_TEXT,
        initial='$DRINKER kicked off a new session on $SITENAME.',
        help_text='Template to use when a session is started.')
    session_joined_template = forms.CharField(max_length=255, widget=WIDE_TEXT,
        initial='$DRINKER joined the session.',
        help_text='Template to use when a session is joined.')
    drink_poured_template = forms.CharField(max_length=255, widget=WIDE_TEXT,
        initial='$DRINKER poured $VOLUME of $BEER on $SITENAME.',
        help_text='Template to use when a drink is poured.')
    user_drink_poured_template = forms.CharField(max_length=255, widget=WIDE_TEXT,
        initial='Just poured $VOLUME of $BEER on $SITENAME. #kegbot',
        help_text='Template to use in user tweets when a drink is poured.')


class UserSettingsForm(forms.Form):
    tweet_session_events = forms.BooleanField(initial=True, required=False,
        help_text='Tweet when you join a session.')
    tweet_drink_events = forms.BooleanField(initial=False, required=False,
        help_text='Tweet every time you pour (caution: potentially annoying).')



# Copyright 2010 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

import copy
import datetime
import pytz

from django.conf import settings
from django.core import urlresolvers
from django.core.urlresolvers import NoReverseMatch
from django.core.urlresolvers import reverse
from django import template
from django.template import Library
from django.template import Node
from django.template import VariableDoesNotExist
from django.template import TemplateSyntaxError
from django.template import Variable
from django.template.defaultfilters import pluralize
from django.utils.safestring import mark_safe

from kegbot.util import kbjson
from kegbot.util import units

from pykeg.core import models
from pykeg.web.charts import charts

register = Library()

@register.inclusion_tag('kegweb/mugshot_box.html', takes_context=True)
def mugshot_box(context, user, boxsize=0):
  c = copy.copy(context)
  c['user'] = user
  c['boxsize'] = boxsize
  return c

@register.inclusion_tag('kegweb/picture-gallery.html', takes_context=True)
def gallery(context, picture_or_pictures, thumb_size='span2', gallery_id=''):
  c = copy.copy(context)
  if not hasattr(picture_or_pictures, '__iter__'):
    c['gallery_pictures'] = [picture_or_pictures]
  else:
    c['gallery_pictures'] = picture_or_pictures
  c['thumb_size'] = thumb_size
  c['gallery_id'] = gallery_id
  return c

@register.inclusion_tag('kegweb/badge.html')
def badge(amount, caption, style='', is_volume=False, do_pluralize=False):
  if is_volume:
    amount = mark_safe(VolumeNode.format(amount, 'mL'))
  if do_pluralize:
    caption += pluralize(amount)
  return {
      'badge_amount': amount,
      'badge_caption': caption,
      'badge_style': style,
  }


### navitem

@register.tag('navitem')
def navitem(parser, token):
  """{% navitem <viewname> <title> [exact] %}"""
  tokens = token.split_contents()
  if len(tokens) < 3:
    raise TemplateSyntaxError, '%s requires at least 3 tokens' % tokens[0]
  return NavitemNode(*tokens[1:])

class NavitemNode(Node):
  def __init__(self, *args):
    self._viewname = args[0]
    self._title = args[1]
    self._exact = 'exact' in args[2:]

  def render(self, context):
    viewname = Variable(self._viewname).resolve(context)
    title = Variable(self._title).resolve(context)
    if viewname.startswith('/'):
      urlbase = viewname
    else:
      urlbase = reverse(viewname)

    request_path = context['request_path']

    if self._exact:
      active = (request_path == urlbase)
    else:
      active = request_path.startswith(urlbase)
    if active:
      res = '<li class="active">'
    else:
      res = '<li>'
    res += '<a href="%s">%s</a></li>' % (urlbase, title)
    return res


### timeago

@register.tag('timeago')
def timeago(parser, token):
  """{% timeago <timestamp> %}"""
  tokens = token.contents.split()
  if len(tokens) != 2:
    raise TemplateSyntaxError, '%s requires 2 tokens' % tokens[0]
  return TimeagoNode(tokens[1])

class TimeagoNode(Node):
  def __init__(self, timestamp_varname):
    self._timestamp_varname = timestamp_varname

  def render(self, context):
    tv = Variable(self._timestamp_varname)
    ts = tv.resolve(context)

    # Try to set time zone information.
    if settings.TIME_ZONE and not settings.USE_TZ:
      try:
        tz = pytz.timezone(settings.TIME_ZONE)
        ts = tz.localize(ts)
      except pytz.UnknownTimeZoneError:
        pass

    iso = ts.isoformat()
    alt = ts.strftime("%A, %B %d, %Y %I:%M%p")
    return '<abbr class="timeago" title="%s">%s</abbr>' % (iso, alt)


### volume

@register.tag('volume')
def volumetag(parser, token):
  """{% volume <amount> %}"""
  tokens = token.contents.split()
  if len(tokens) < 2:
    raise TemplateSyntaxError, '%s requires at least 2 tokens' % tokens[0]
  return VolumeNode(tokens[1], tokens[2:])

class VolumeNode(Node):
  TEMPLATE = """
    <span class="hmeasure" title="%(title)s">
      <span class="num">%(amount)s</span>
      <span class="unit">%(units)s</span>
    </span>""".strip()

  def __init__(self, volume_varname, extra_args):
    self._volume_varname = volume_varname
    self._extra_args = extra_args

  def render(self, context):
    tv = Variable(self._volume_varname)
    try:
      num = float(tv.resolve(context))
    except (VariableDoesNotExist, ValueError):
      num = 'unknown'
    unit = 'mL'
    return self.format(num, unit)

  @classmethod
  def format(cls, amount, units):
    title = '%s %s' % (amount, units)
    return cls.TEMPLATE % vars()

### drinker
@register.tag('drinker_name')
def drinker_name_tag(parser, token):
  """{% drinker_name <drink_or_user_obj> [nolink] %}"""
  tokens = token.contents.split()
  if len(tokens) < 2:
    raise TemplateSyntaxError, '%s requires at least 2 tokens' % tokens[0]
  return DrinkerNameNode(tokens[1], tokens[2:])

class DrinkerNameNode(Node):
  def __init__(self, drink_varname, extra_args):
    self._varname = drink_varname
    self._extra_args = extra_args

  def render(self, context):
    obj = Variable(self._varname)
    try:
      obj = obj.resolve(context)
    except (VariableDoesNotExist, ValueError):
      obj = None

    user = None
    if obj:
      if isinstance(obj, models.Drink) or isinstance(obj, models.SystemEvent):
        user = obj.user
      elif isinstance(obj, models.User):
        user = obj
    if user:
      if 'nolink' in self._extra_args:
        return user.username
      else:
        return '<a href="%s">%s</a>' % (reverse('kb-drinker', args=[user.username]), user.username)
    return context['guest_info']['name']

  @classmethod
  def format(cls, amount, units):
    title = '%s %s' % (amount, units)
    return cls.TEMPLATE % vars()


### chart

@register.tag('chart')
def chart(parser, tokens):
  """{% chart <charttype> <obj> width height %}"""
  tokens = tokens.contents.split()
  if len(tokens) < 4:
    raise TemplateSyntaxError('chart requires at least 4 arguments')
  charttype = tokens[1]
  try:
    width = int(tokens[-2])
    height = int(tokens[-1])
  except ValueError:
    raise TemplateSyntaxError('invalid width or height')
  args = tokens[2:-2]
  return ChartNode(charttype, width, height, args)

class ChartNode(Node):
  CHART_TMPL = '''
  <!-- begin chart %(chart_id)s -->
  <div id="chart-%(chart_id)s-container"
      style="height: %(height)spx; width: %(width)spx;"
      class="kb-chartbox"></div>
  <script type="text/javascript">
    var chart_%(chart_id)s;
    $(document).ready(function() {
      var chart_data = %(chart_data)s;
      chart_%(chart_id)s = new Highcharts.Chart(chart_data);
    });
  </script>
  <!-- end chart %(chart_id)s -->

  '''
  ERROR_TMPL = '''
  <!-- begin chart %(chart_id)s -->
  <div id="chart-%(chart_id)s-container"
      style="height: %(height)spx; width: %(width)spx;"
      class="kb-chartbox-error">
    %(error_str)s
  </div>
  <!-- end chart %(chart_id)s -->
  '''
  def __init__(self, charttype, width, height, args):
    self._charttype = charttype
    self._width = width
    self._height = height
    self._args = args

    try:
      self._chart_fn = getattr(self, 'chart_%s' % (self._charttype,))
    except AttributeError:
      raise TemplateSyntaxError('unknown chart type: %s' % self._charttype)

  def _get_chart_id(self, context):
    # TODO(mikey): Is there a better way to store _CHART_ID?
    if not hasattr(context, '_CHART_ID'):
      context._CHART_ID = 0
    context._CHART_ID += 1
    return context._CHART_ID

  def render(self, context):
    chart_id = self._get_chart_id(context)

    width = self._width
    height = self._height

    obj = Variable(self._args[0]).resolve(context)
    try:
      chart_result = self._chart_fn(obj)
    except charts.ChartError, e:
      error_str = 'chart error: %s' % (e,)
      return ChartNode.ERROR_TMPL % vars()
    chart_base = {
      'chart': {
        'borderColor': '#eeeeff',
        'borderWidth': 0,
        'renderTo': 'chart-%s-container' % chart_id,
      },
      'credits': {
        'enabled': False,
      },
      'legend': {
        'enabled': False,
      },
      'margin': [0, 0, 0, 0],
      'title': {
        'text': None,
      },
      'yAxis': {
        'labels': {
          'align': 'left'
        },
        'title': {
          'text': None,
        }
      },
    }

    chart_data = chart_base
    for k, v in chart_result.iteritems():
      if k not in chart_data:
        chart_data[k] = v
      elif type(v) == type({}):
        chart_data[k].update(v)
      else:
        chart_data[k] = v
    chart_data = kbjson.dumps(chart_data, indent=None)
    return ChartNode.CHART_TMPL % vars()

  def chart_sensor(self, obj):
    """Shows a simple line plot of a specific temperature sensor.

    Args:
      obj - the models.ThermoSensor to plot
    """
    return charts.TemperatureSensorChart(obj)

  def chart_keg_volume(self, obj):
    """Shows a horizontal bar chart of keg served/remaining volume.

    Args:
      obj - the models.Keg instance to chart
    """
    return charts.KegVolumeChart(obj)

  def chart_volume_by_day(self, obj):
    """Shows keg or session usage by day of the week.

    Args:
      obj - the stats instance to chart
    """
    return charts.VolumeByWeekday(obj)

  def chart_sessions_weekday(self, obj):
    """Vertical bar chart showing session volume by day of week.

    Args:
      obj - an iterable of models.DrinkingSession or
            models.UserDrinkingSessionPart instances
    """
    return charts.UserSessionsByWeekday(obj)

  def chart_sessions_volume(self, obj):
    """Line chart showing session volumes.

    Args:
      obj - an iterable of models.DrinkingSession or
            models.UserDrinkingSessionPart instances
    """
    return charts.SessionVolumes(obj)

  def chart_users_by_volume(self, obj):
    """Pie chart showing users by volume.

    Args:
      obj - the stats instance to chart
    """
    return charts.UsersByVolume(obj)

  def chart_user_session_chunks(self, obj):
    """Show's a single user's activity within a session.

    Args:
      obj - the models.UserSessionChunk for the user/session to chart
    """
    return charts.UserSessionChunks(obj)

@register.filter
def volume(text, fmt='pints'):
  try:
    vol = units.Quantity(float(text))
  except ValueError:
    return text
  if fmt == 'pints':
    res = vol.InPints()
  elif fmt == 'liters':
    res = vol.InLiters()
  elif fmt == 'ounces':
    res = vol.InOunces()
  elif fmt == 'gallons':
    res = vol.InUSGallons()
  elif fmt == 'twelveounces':
    res = vol.InTwelveOunceBeers()
  elif fmt == 'halfbarrels':
    res = vol.InHalfBarrelKegs()
  else:
    raise TemplateSyntaxError, 'Unknown volume format: %s' % fmt
  return float(res)



# -*- coding: latin-1 -*-
# Copyright 2009 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

from django.contrib import admin

from kegbot.util import util

from pykeg.core import models

admin.site.register(models.UserProfile)

admin.site.register(models.KegSize)

class KegbotSiteAdmin(admin.ModelAdmin):
  list_display = ('name', 'is_active')
  list_filter = ('is_active',)
admin.site.register(models.KegbotSite, KegbotSiteAdmin)

class KegTapAdmin(admin.ModelAdmin):
  list_display = ('name', 'meter_name', 'relay_name', 'current_keg')
admin.site.register(models.KegTap, KegTapAdmin)

class KegAdmin(admin.ModelAdmin):
  list_display = ('id', 'type', 'online')
  list_filter = ('online', )
  search_fields = ('id', 'type__name')
admin.site.register(models.Keg, KegAdmin)

class DrinkAdmin(admin.ModelAdmin):
  list_display = ('id', 'user', 'keg', 'time')
  list_filter = ('keg', 'time')
  search_fields = ('id', 'user__username')
admin.site.register(models.Drink, DrinkAdmin)

class AuthenticationTokenAdmin(admin.ModelAdmin):
  list_display = ('auth_device', 'user', 'token_value', 'nice_name', 'enabled', 'IsActive')
  list_filter = ('auth_device', 'enabled')
  search_fields = ('user__username', 'token_value', 'nice_name')
admin.site.register(models.AuthenticationToken, AuthenticationTokenAdmin)

class DrinkingSessionAdmin(admin.ModelAdmin):
  list_display = ('id', 'start_time', 'end_time', 'volume_ml', 'GetTitle')
  list_filter = ('start_time',)
  search_fields = ('name',)
admin.site.register(models.DrinkingSession, DrinkingSessionAdmin)

class ThermoSensorAdmin(admin.ModelAdmin):
  list_display = ('raw_name', 'nice_name')
  search_fields = list_display
admin.site.register(models.ThermoSensor, ThermoSensorAdmin)

def thermolog_deg_c(obj):
  return '%.2f C' % (obj.temp,)

def thermolog_deg_f(obj):
  return '%.2f F' % (util.CtoF(obj.temp),)

class ThermologAdmin(admin.ModelAdmin):
  list_display = ('sensor', thermolog_deg_c, thermolog_deg_f, 'time')
  list_filter = ('sensor', 'time')

admin.site.register(models.Thermolog, ThermologAdmin)

class SystemEventAdmin(admin.ModelAdmin):
  list_display = ('id', 'kind', 'time', 'user', 'drink', 'keg', 'session')
  list_filter = ('kind', 'time')
admin.site.register(models.SystemEvent, SystemEventAdmin)

class PictureAdmin(admin.ModelAdmin):
  list_display = ('id', 'time')
  list_filter = ('time',)
admin.site.register(models.Picture, PictureAdmin)

class PourPictureAdmin(admin.ModelAdmin):
  list_display = ('id', 'time', 'user', 'drink', 'keg', 'session', 'caption')
  list_filter = ('time',)
admin.site.register(models.PourPicture, PourPictureAdmin)

class BeerTypeInline(admin.TabularInline):
  model = models.BeerType
  fieldsets = (
    (None, {
        'fields': ('name', 'brewer', 'style', 'abv', 'image', 'untappd_beer_id')
    }),
  )

class BrewerAdmin(admin.ModelAdmin):
  list_display = ('name', 'country', 'added', 'edited')
  ordering = ('name',)
  search_fields = ('name',)
  inlines = [
      BeerTypeInline,
  ]
  fieldsets = (
    (None, {
        'fields': ('name', 'country', 'origin_state', 'origin_city', 'image')
    }),
    ('Advanced options', {
        'classes': ('collapse',),
        'fields': ('production', 'url', 'description')
    }),
  )
admin.site.register(models.Brewer, BrewerAdmin)

class BeerStyleAdmin(admin.ModelAdmin):
  list_display = ('name', 'added', 'edited')
  model = models.BeerStyle
  ordering = ('name',)
  search_fields = ('name',)
admin.site.register(models.BeerStyle, BeerStyleAdmin)

class BeerTypeAdmin(admin.ModelAdmin):
  list_display = ('name', 'brewer', 'style', 'added', 'edited')
  ordering = ('brewer__name', 'name')
  search_fields = ('name', 'brewer__name')
  fieldsets = (
    (None, {
        'fields': ('name', 'brewer', 'style', 'abv', 'image')
    }),
    ('Advanced options', {
        'classes': ('collapse',),
        'fields': ('calories_oz', 'carbs_oz', 'original_gravity',
            'specific_gravity', 'beerdb_id', 'untappd_beer_id')
    }),
  )

admin.site.register(models.BeerType, BeerTypeAdmin)



# Copyright 2013 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

"""Celery tasks for Twitter."""

import tweepy
from celery.task import task

from pykeg.plugin import util

logger = util.get_logger(__name__)

@task(expires=60)
def send_tweet(consumer_key, consumer_secret, oauth_token, oauth_token_secret, tweet):
    auth = tweepy.OAuthHandler(consumer_key, consumer_secret)
    auth.set_access_token(oauth_token, oauth_token_secret)
    api = tweepy.API(auth)
    logger.info('Sending tweet: {}'.format(repr(tweet)))
    result = api.update_status(tweet)
    return result



#!/usr/bin/env python
#
# Copyright 2010 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

import cStringIO
import datetime

from django.contrib import messages
from django.contrib.admin.views.decorators import staff_member_required
from django.contrib.auth.decorators import login_required
from django.core.urlresolvers import reverse
from django.db.models import Q
from django.http import Http404
from django.http import HttpResponse
from django.shortcuts import get_object_or_404
from django.shortcuts import redirect
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.forms import widgets
from django.views.decorators.http import require_POST
from django.utils import timezone
from django.core.paginator import Paginator, EmptyPage, PageNotAnInteger
from django.views.decorators.http import require_http_methods

from kegbot.util import kbjson
from kegbot.util import units

from pykeg.core import backend
from pykeg.core import backup
from pykeg.core import logger
from pykeg.core import models

from pykeg.web.kegadmin import forms

@staff_member_required
def dashboard(request):
  context = RequestContext(request)
  recent_time = timezone.now() - datetime.timedelta(days=30)
  active_users = models.User.objects.filter(is_active=True)
  new_users = models.User.objects.filter(date_joined__gte=recent_time)
  context['num_users'] = len(active_users)
  context['num_new_users'] = len(new_users)
  return render_to_response('kegadmin/dashboard.html', context_instance=context)

@staff_member_required
def general_settings(request):
  context = RequestContext(request)
  settings = models.SiteSettings.get()
  form = forms.SiteSettingsForm(instance=settings)
  if request.method == 'POST':
    form = forms.SiteSettingsForm(request.POST, instance=settings)
    if form.is_valid():
      form.save()
      messages.success(request, 'Site settings were successfully updated.')
  context['settings_form'] = form
  return render_to_response('kegadmin/index.html', context_instance=context)

@staff_member_required
def tap_list(request):
  context = RequestContext(request)
  context['taps'] = models.KegTap.objects.all()
  return render_to_response('kegadmin/tap_list.html', context_instance=context)

@staff_member_required
def add_tap(request):
  context = RequestContext(request)
  form = forms.TapForm(site=request.kbsite)
  if request.method == 'POST':
    form = forms.TapForm(request.POST, site=request.kbsite)
    if form.is_valid():
      new_tap = form.save(commit=False)
      new_tap.site = request.kbsite
      new_tap.save()
      messages.success(request, 'Tap created.')
      return redirect('kegadmin-taps')
  context['form'] = form
  return render_to_response('kegadmin/add_tap.html', context_instance=context)

@staff_member_required
def tap_detail(request, tap_id):
  tap = get_object_or_404(models.KegTap, id=tap_id)

  record_drink_form = forms.RecordDrinkForm()
  activate_keg_form = forms.ChangeKegForm()
  tap_settings_form = forms.TapForm(instance=tap, site=request.kbsite)

  if request.method == 'POST':
    if 'submit_change_keg_form' in request.POST:
      activate_keg_form = forms.ChangeKegForm(request.POST)
      if activate_keg_form.is_valid():
        activate_keg_form.save(tap)
        messages.success(request, 'The new keg was activated. Bottoms up!')
        return redirect('kegadmin-taps')

    elif 'submit_tap_form' in request.POST:
      tap_settings_form = forms.TapForm(request.POST, instance=tap, site=request.kbsite)
      if tap_settings_form.is_valid():
        tap_settings_form.save()
        messages.success(request, 'Tap settings saved.')
        tap_settings_form = forms.TapForm(instance=tap, site=request.kbsite)

    elif 'submit_delete_tap_form' in request.POST:
      delete_form = forms.DeleteTapForm(request.POST)
      if delete_form.is_valid():
        if tap.current_keg:
          request.backend.EndKeg(tap)
        tap.delete()
        messages.success(request, 'Tap deleted.')
        return redirect('kegadmin-taps')

    elif 'submit_end_keg_form' in request.POST:
      end_keg_form = forms.EndKegForm(request.POST)
      if end_keg_form.is_valid():
        old_keg = request.backend.EndKeg(tap)
        messages.success(request, 'Keg %s was ended.' % old_keg.id)

    elif 'submit_record_drink' in request.POST:
      record_drink_form = forms.RecordDrinkForm(request.POST)
      if record_drink_form.is_valid():
        user = record_drink_form.cleaned_data.get('user')
        volume_ml = record_drink_form.cleaned_data.get('volume_ml')
        d = request.backend.RecordDrink(tap.meter_name, ticks=0, username=user,
            volume_ml=volume_ml)
        messages.success(request, 'Drink %s recorded.' % d.id)
      else:
        messages.error(request, 'Please enter a valid volume and user.')

    else:
      messages.warning(request, 'No form data was found. Bug?')

  end_keg_form = forms.EndKegForm(initial={'keg':tap.current_keg})

  context = RequestContext(request)
  context['tap'] = tap
  context['current_keg'] = tap.current_keg
  context['activate_keg_form'] = activate_keg_form
  context['record_drink_form'] = record_drink_form
  context['end_keg_form'] = end_keg_form
  context['tap_settings_form'] = tap_settings_form
  context['delete_tap_form'] = forms.DeleteTapForm()
  return render_to_response('kegadmin/tap_detail.html', context_instance=context)

@staff_member_required
def user_list(request):
  context = RequestContext(request)

  if request.method == 'POST':
    form = forms.FindUserForm(request.POST)
    if form.is_valid():
      username = form.cleaned_data.get('username')
      try:
        user = models.User.objects.get(username=username)
        return redirect('kegadmin-edit-user', user.id)
      except models.User.DoesNotExist:
        messages.error(request, 'User "%s" does not exist.' % username)

  users = models.User.objects.all().order_by('-id')
  paginator = Paginator(users, 25)

  page = request.GET.get('page')
  try:
    users = paginator.page(page)
  except PageNotAnInteger:
    users = paginator.page(1)
  except EmptyPage:
    users = paginator.page(paginator.num_pages)

  context['users'] = users
  return render_to_response('kegadmin/user_list.html', context_instance=context)

@staff_member_required
def add_user(request):
  context = RequestContext(request)
  form = forms.UserForm()
  if request.method == 'POST':
    form = forms.UserForm(request.POST)
    if form.is_valid():
      instance = form.save(commit=False)
      instance.set_password(form.cleaned_data.get('password'))
      instance.save()
      messages.success(request, 'User "%s" created.' % instance.username)
      return redirect('kegadmin-users')
  context['form'] = form
  return render_to_response('kegadmin/add_user.html', context_instance=context)

@staff_member_required
def user_detail(request, user_id):
  user = get_object_or_404(models.User, id=user_id)

  if request.method == 'POST':
    if 'submit_enable' in request.POST:
      if user.is_active:
        messages.error(request, 'User is already enabled.')
      else:
        user.is_active = True
        user.save()
        messages.success(request, 'User %s was enabled.' % user.username)

    elif 'submit_disable' in request.POST:
      if not user.is_active:
        messages.error(request, 'User is already disabled.')
      else:
        user.is_active = False
        user.save()
        messages.success(request, 'User %s was disabled.' % user.username)

  context = RequestContext(request)
  context['user'] = user
  context['profile'] = user.get_profile()
  context['stats'] = context['profile'].GetStats()

  context['tokens'] = user.tokens.all().order_by('created_time')

  return render_to_response('kegadmin/user_detail.html', context_instance=context)

@staff_member_required
def drink_list(request):
  context = RequestContext(request)
  drinks = models.Drink.objects.all().order_by('-time')
  paginator = Paginator(drinks, 25)

  page = request.GET.get('page')
  try:
    drinks = paginator.page(page)
  except PageNotAnInteger:
    drinks = paginator.page(1)
  except EmptyPage:
    drinks = paginator.page(paginator.num_pages)

  context['drinks'] = drinks
  return render_to_response('kegadmin/drink_list.html', context_instance=context)


@staff_member_required
@require_http_methods(["POST"])
def drink_edit(request, drink_id):
  drink = get_object_or_404(models.Drink, id=drink_id)

  if 'submit_cancel' in request.POST:
    form = forms.CancelDrinkForm(request.POST)
    old_keg = drink.keg
    if form.is_valid():
      request.backend.CancelDrink(drink)
      messages.success(request, 'Drink %s was cancelled.' % drink_id)
      return redirect(old_keg.get_absolute_url())
    else:
      messages.error(request, 'Invalid request')
      return redirect(drink.get_absolute_url())

  elif 'submit_reassign' in request.POST:
    form = forms.ReassignDrinkForm(request.POST)
    if form.is_valid():
      username = form.cleaned_data.get('username', None)
      try:
        request.backend.AssignDrink(drink, username)
        messages.success(request, 'Drink %s was reassigned.' % drink_id)
      except models.User.DoesNotExist:
        messages.error(request, 'No such user')
    else:
      messages.error(request, 'Invalid request')
    return redirect(drink.get_absolute_url())

  elif 'submit_edit_volume' in request.POST:
    form = forms.ChangeDrinkVolumeForm(request.POST)
    if form.is_valid():
      volume_ml = form.cleaned_data.get('volume_ml')
      if volume_ml == drink.volume_ml:
        messages.warning(request, 'Drink volume unchanged.')
      else:
        request.backend.SetDrinkVolume(drink, volume_ml)
        messages.success(request, 'Drink %s was updated.' % drink_id)
    else:
      messages.error(request, 'Please provide a valid volume.')
    return redirect(drink.get_absolute_url())

  message.error(request, 'Unknown action.')
  return redirect(drink.get_absolute_url())


@staff_member_required
def token_list(request):
  context = RequestContext(request)
  tokens = models.AuthenticationToken.objects.all().order_by('-created_time')
  paginator = Paginator(tokens, 25)

  page = request.GET.get('page')
  try:
    tokens = paginator.page(page)
  except PageNotAnInteger:
    tokens = paginator.page(1)
  except EmptyPage:
    tokens = paginator.page(paginator.num_pages)

  context['tokens'] = tokens
  return render_to_response('kegadmin/token_list.html', context_instance=context)


@staff_member_required
def token_detail(request, token_id):
  token = get_object_or_404(models.AuthenticationToken, id=token_id)

  username = ''
  if token.user:
    username = token.user.username

  form = forms.TokenForm(instance=token, initial={'username': username})
  if request.method == 'POST':
    form = forms.TokenForm(request.POST, instance=token)
    if form.is_valid():
      instance = form.save(commit=False)
      instance.user = form.cleaned_data['user']
      instance.save()
      messages.success(request, 'Token updated.')

  context = RequestContext(request)
  context['token'] = token
  context['form'] = form
  return render_to_response('kegadmin/token_detail.html', context_instance=context)

@staff_member_required
def add_token(request):
  context = RequestContext(request)
  form = forms.AddTokenForm()
  if request.method == 'POST':
    form = forms.AddTokenForm(request.POST)
    if form.is_valid():
      instance = form.save(commit=False)
      instance.user = form.cleaned_data['user']
      instance.site = request.kbsite
      instance.save()
      messages.success(request, 'Token created.')
      return redirect('kegadmin-tokens')
  context['form'] = form
  return render_to_response('kegadmin/add_token.html', context_instance=context)


@staff_member_required
def beer_type_list(request):
  context = RequestContext(request)
  beers = models.BeerType.objects.all().order_by('name')
  paginator = Paginator(beers, 25)

  page = request.GET.get('page')
  try:
    beers = paginator.page(page)
  except PageNotAnInteger:
    beers = paginator.page(1)
  except EmptyPage:
    beers = paginator.page(paginator.num_pages)

  context['beers'] = beers
  return render_to_response('kegadmin/beer_type_list.html', context_instance=context)


@staff_member_required
def beer_type_detail(request, beer_id):
  btype = get_object_or_404(models.BeerType, id=beer_id)

  form = forms.BeerTypeForm(instance=btype)
  if request.method == 'POST':
    form = forms.BeerTypeForm(request.POST, instance=btype)
    if form.is_valid():
      btype = form.save()
      new_image = request.FILES.get('new_image')
      if new_image:
        pic = models.Picture.objects.create()
        pic.image.save(new_image.name, new_image)
        pic.save()
        btype.image = pic
        btype.save()

      messages.success(request, 'Beer type updated.')

  context = RequestContext(request)
  context['beer_type'] = btype
  context['form'] = form
  return render_to_response('kegadmin/beer_type_detail.html', context_instance=context)


@staff_member_required
def brewer_list(request):
  context = RequestContext(request)
  brewers = models.Brewer.objects.all().order_by('name')
  paginator = Paginator(brewers, 25)

  page = request.GET.get('page')
  try:
    brewers = paginator.page(page)
  except PageNotAnInteger:
    brewers = paginator.page(1)
  except EmptyPage:
    brewers = paginator.page(paginator.num_pages)

  context['brewers'] = brewers
  return render_to_response('kegadmin/brewer_list.html', context_instance=context)

@staff_member_required
def brewer_detail(request, brewer_id):
  brewer = get_object_or_404(models.Brewer, id=brewer_id)

  form = forms.BrewerForm(instance=brewer)
  if request.method == 'POST':
    form = forms.BrewerForm(request.POST, instance=brewer)
    if form.is_valid():
      form.save()
      messages.success(request, 'Brewer updated.')

  context = RequestContext(request)
  context['brewer'] = brewer
  context['form'] = form
  return render_to_response('kegadmin/brewer_detail.html', context_instance=context)


@staff_member_required
def beer_style_list(request):
  context = RequestContext(request)
  styles = models.BeerStyle.objects.all().order_by('name')
  paginator = Paginator(styles, 25)

  page = request.GET.get('page')
  try:
    styles = paginator.page(page)
  except PageNotAnInteger:
    styles = paginator.page(1)
  except EmptyPage:
    styles = paginator.page(paginator.num_pages)

  context['styles'] = styles
  return render_to_response('kegadmin/beer_style_list.html', context_instance=context)

@staff_member_required
def beer_style_detail(request, style_id):
  style = get_object_or_404(models.BeerStyle, id=style_id)

  form = forms.BeerStyleForm(instance=style)
  if request.method == 'POST':
    form = forms.BeerStyleForm(request.POST, instance=style)
    if form.is_valid():
      form.save()
      messages.success(request, 'Beer style updated.')

  context = RequestContext(request)
  context['style'] = style
  context['form'] = form
  return render_to_response('kegadmin/beer_style_detail.html', context_instance=context)

@staff_member_required
def autocomplete_beer_type(request):
  context = RequestContext(request)
  search = request.GET.get('q')
  if search:
    types = models.BeerType.objects.filter(Q(name__icontains=search) | Q(brewer__name__icontains=search))
  else:
    types = models.BeerType.objects.all()
  types = types[:10]  # autocomplete widget limited to 10
  values = []
  for beer in types:
    values.append({
      'name': beer.name,
      'id': beer.id,
      'brewer_name': beer.brewer.name,
      'brewer_id': beer.brewer.id,
      'style_name': beer.style.name,
      'style_id': beer.style.id,
  })
  return HttpResponse(kbjson.dumps(values, indent=None),
    mimetype='application/json', status=200)

@staff_member_required
def autocomplete_user(request):
  context = RequestContext(request)
  search = request.GET.get('q')
  if search:
    users = models.User.objects.filter(Q(username__icontains=search) | Q(email__icontains=search))
  else:
    users = models.User.objects.all()
  users = users[:10]  # autocomplete widget limited to 10
  values = []
  for user in users:
    values.append({
      'username': user.username,
      'id': user.id,
      'email': user.email,
      'is_active': user.is_active,
  })
  return HttpResponse(kbjson.dumps(values, indent=None),
    mimetype='application/json', status=200)

@staff_member_required
def autocomplete_token(request):
  context = RequestContext(request)
  search = request.GET.get('q')
  if search:
    tokens = models.AuthenticationToken.objects.filter(
        Q(token_value__icontains=search) | Q(nice_name__icontains=search))
  else:
    tokens = models.AuthenticationToken.objects.all()
  tokens = tokens[:10]  # autocomplete widget limited to 10
  values = []
  for token in tokens:
    values.append({
      'username': token.user.username,
      'id': token.id,
      'auth_device': token.auth_device,
      'token_value': token.token_value,
      'enabled': token.enabled,
  })
  return HttpResponse(kbjson.dumps(values, indent=None),
    mimetype='application/json', status=200)

@staff_member_required
def plugin_settings(request, plugin_name):
  context = RequestContext(request)
  plugin = request.plugins.get(plugin_name, None)
  if not plugin:
    raise Http404('Plugin "%s" not loaded' % plugin_name)

  view = plugin.get_admin_settings_view()
  if not view:
    raise Http404('No settings for this plugin')

  return view(request, plugin)

@staff_member_required
def backup_restore(request):
  context = RequestContext(request)
  return render_to_response('kegadmin/backup-restore.html', context_instance=context)

@staff_member_required
def generate_backup(request):
  context = RequestContext(request)

  fmt = request.GET.get('fmt', 'minjson')
  if fmt == 'json':
    indent = 2
  else:
    indent = None
    fmt = 'minjson'

  kbsite = request.kbsite
  datestr = timezone.now().strftime('%Y%m%d-%H%M%S')
  filename = 'kegbot-%s.%s.%s.txt' % (kbsite.name, datestr, fmt)

  output_fp = cStringIO.StringIO()
  backup.dump(output_fp, kbsite, indent=indent)

  response = HttpResponse(output_fp.getvalue(),
      mimetype="application/octet-stream")
  response['Content-Disposition'] = 'attachment; filename=%s' % filename
  output_fp.close()
  return response

@staff_member_required
def logs(request):
  context = RequestContext(request)
  context['errors'] = logger.get_cached_logs()
  return render_to_response('kegadmin/logs.html', context_instance=context)




# Kegbot local settings, for travis-ci.org build

# NEVER set DEBUG to `True` in production.
import os

HOME = os.environ['HOME']

DEBUG = True
TEMPLATE_DEBUG = DEBUG

DATABASES = {'default': {'ENGINE': 'django.db.backends.sqlite3', 'NAME': HOME + '/kegbot-data/kegbot.sqlite'}}

KEGBOT_ROOT = HOME + '/kegbot-data'

MEDIA_ROOT = KEGBOT_ROOT  + '/media'

STATIC_ROOT = KEGBOT_ROOT + '/static'

TIME_ZONE = 'America/Los_Angeles'

CACHES = {'default': {'LOCATION': '127.0.0.1:11211', 'BACKEND': 'django.core.cache.backends.memcached.MemcachedCache'}}

SECRET_KEY = 'testkey'




from django.conf.urls import patterns
from django.conf.urls import url

from pykeg.core import features
from pykeg.web.account.views import password_change
from pykeg.web.account.views import password_change_done


urlpatterns = patterns('pykeg.web.account.views',
    url(r'^$', 'account_main', name='kb-account-main'),
    url(r'^password/done/$', password_change_done, name='password_change_done'),
    url(r'^password/$', password_change, name='password_change'),
    url(r'^mugshot/$', 'edit_mugshot', name='account-mugshot'),
    url(r'^regenerate-api-key/$', 'regenerate_api_key', name='regen-api-key'),
    url(r'^plugin/(?P<plugin_name>\w+)/$', 'plugin_settings', name='account-plugin-settings'),
)

if features.use_facebook():
  urlpatterns += patterns('',
      url(r'fb-settings/$', 'pykeg.web.contrib.facebook.views.account_settings', name='fb-account-settings'),
  )

from pykeg.plugin import util
urlpatterns += util.get_account_urls()



# Copyright 2011 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

from django.contrib.auth.signals import user_logged_in
from django.contrib.auth.signals import user_logged_out
from django.contrib import messages

def on_logged_in(sender, user, request, **kwargs):
  messages.add_message(request, messages.INFO, 'You are now logged in!',
    fail_silently=True)
user_logged_in.connect(on_logged_in)

def on_logged_out(sender, user, request, **kwargs):
  messages.add_message(request, messages.INFO, 'You have been logged out.',
    fail_silently=True)
user_logged_out.connect(on_logged_out)



# Copyright 2013 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

from django.contrib import messages
from django.core.urlresolvers import reverse
from django.shortcuts import redirect
from socialregistration.clients.oauth import OAuthError
from socialregistration.contrib.twitter.client import Twitter
from django.contrib.admin.views.decorators import staff_member_required
from django.contrib.auth.decorators import login_required
from django.shortcuts import render_to_response
from django.template import RequestContext
import oauth2 as oauth

from . import forms

class TwitterClient(Twitter):
  def set_callback_url(self, url):
    self.callback_url = url

  def get_callback_url(self):
    return self.callback_url


@staff_member_required
def admin_settings(request, plugin):
  context = RequestContext(request)

  consumer_key, consumer_secret = plugin.get_credentials()
  initial = {
    'consumer_key': consumer_key or '',
    'consumer_secret': consumer_secret or '',
  }

  credentials_form = forms.CredentialsForm(initial=initial)
  settings_form = forms.SiteSettingsForm()
  plugin.load_form_defaults(settings_form, 'site_settings')

  if request.method == 'POST':
    if 'submit-keys' in request.POST:
      credentials_form = forms.CredentialsForm(request.POST)
      if credentials_form.is_valid():
        consumer_key = credentials_form.cleaned_data['consumer_key']
        consumer_secret = credentials_form.cleaned_data['consumer_secret']
        plugin.set_credentials(consumer_key, consumer_secret)
        messages.success(request, 'Keys updated')

    elif 'submit-settings' in request.POST:
      settings_form = forms.SiteSettingsForm(request.POST)
      if settings_form.is_valid():
        plugin.save_form(settings_form, 'site_settings')
        messages.success(request, 'Settings updated')

  context['have_credentials'] = consumer_key and consumer_secret
  context['plugin'] = plugin
  context['site_profile'] = plugin.get_site_profile()
  context['credentials_form'] = credentials_form
  context['settings_form'] = settings_form

  return render_to_response('contrib/twitter/admin_settings.html', context_instance=context)


@login_required
def user_settings(request, plugin):
  context = RequestContext(request)
  user = request.user

  consumer_key, consumer_secret = plugin.get_credentials()

  settings_form = plugin.get_user_settings_form(user)

  if request.method == 'POST':
    if 'submit-settings' in request.POST:
      settings_form = forms.UserSettingsForm(request.POST)
      if settings_form.is_valid():
        plugin.save_user_settings_form(user, settings_form)
        messages.success(request, 'Settings updated')

  context['have_credentials'] = consumer_key and consumer_secret
  context['plugin'] = plugin
  context['profile'] = plugin.get_user_profile(user)
  context['settings_form'] = settings_form

  return render_to_response('contrib/twitter/twitter_user_settings.html', context_instance=context)


@staff_member_required
def site_twitter_redirect(request):
  if 'submit-remove' in request.POST:
    plugin = request.plugins.get('twitter')
    plugin.remove_site_profile()
    messages.success(request, 'Removed Twitter account.')
    return redirect('kegadmin-plugin-settings', plugin_name='twitter')

  plugin = request.plugins['twitter']

  consumer_key, consumer_secret = plugin.get_credentials()
  client = TwitterClient()
  client.consumer = oauth.Consumer(consumer_key, consumer_secret)

  url = request.kbsite.settings.reverse_full('plugin-twitter-site_twitter_callback')
  client.set_callback_url(url)

  request.session['site_twitter'] = client

  try:
    return redirect(client.get_redirect_url())
  except OAuthError, error:
    messages.error(request, 'Error: %s' % str(error))
    return redirect('kegadmin-plugin-settings', plugin_name='twitter')


@staff_member_required
def site_twitter_callback(request):
  try:
    client = request.session['site_twitter']
    del request.session['site_twitter']
    token = client.complete(dict(request.GET.items()))
  except KeyError:
    messages.error(request, 'Session expired.')
  except OAuthError, error:
    messages.error(request, str(error))
  else:
    user_info = client.get_user_info()
    plugin = request.plugins.get('twitter')
    plugin.save_site_profile(token.key, token.secret, user_info['screen_name'],
        int(user_info['user_id']))
    messages.success(request, 'Successfully linked to @%s' % user_info['screen_name'])

  return redirect('kegadmin-plugin-settings', plugin_name='twitter')


@login_required
def user_twitter_redirect(request):
  if 'submit-remove' in request.POST:
    plugin = request.plugins.get('twitter')
    plugin.remove_user_profile(request.user)
    messages.success(request, 'Removed Twitter account.')
    return redirect('account-plugin-settings', plugin_name='twitter')

  plugin = request.plugins['twitter']

  consumer_key, consumer_secret = plugin.get_credentials()
  client = TwitterClient()
  client.consumer = oauth.Consumer(consumer_key, consumer_secret)

  url = request.kbsite.settings.reverse_full('plugin-twitter-user_twitter_callback')
  client.set_callback_url(url)

  request.session['user_twitter'] = client

  try:
    return redirect(client.get_redirect_url())
  except OAuthError, error:
    messages.error(request, 'Error: %s' % str(error))
    return redirect('account-plugin-settings', plugin_name='twitter')


@staff_member_required
def user_twitter_callback(request):
  try:
    client = request.session['user_twitter']
    del request.session['user_twitter']
    token = client.complete(dict(request.GET.items()))
  except KeyError:
    messages.error(request, 'Session expired.')
  except OAuthError, error:
    messages.error(request, str(error))
  else:
    user_info = client.get_user_info()
    plugin = request.plugins.get('twitter')
    plugin.save_user_profile(request.user, token.key, token.secret,
        user_info['screen_name'], int(user_info['user_id']))
    messages.success(request, 'Successfully linked to @%s' % user_info['screen_name'])

  return redirect('account-plugin-settings', plugin_name='twitter')



# Copyright 2012 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

from django.conf import settings
from django.http import HttpResponse

from . import util

import logging
import sys

LOGGER = logging.getLogger(__name__)

class ApiRequestMiddleware:
  def process_view(self, request, view_func, view_args, view_kwargs):
    request.is_kb_api_request = util.is_api_request(request)
    if not request.is_kb_api_request:
      # Not an API request. Skip me.
      return None

    try:
      if request.need_setup:
        raise ValueError('Setup required')
      elif request.need_upgrade:
        raise ValueError('Upgrade required')

      need_auth = util.needs_auth(view_func)
      privacy = request.kbsite.settings.privacy
      if request.path in ('/api/login/', '/api/get-api-key/'):
        # API request to whitelisted path.
        need_auth = False
      else:
        # API request to non-whitelisted path, in non-public site privacy mode.
        # Demand API key.
        if privacy == 'members' and not request.user.is_authenticated():
          need_auth = True
        elif privacy == 'staff' and not request.user.is_staff:
          need_auth = True

      if need_auth:
        util.check_api_key(request)

      if request.method == 'GET':
        cached = request.kbcache.gen_get(cache_key(request))
        if cached:
          response = util.build_response(request, cached, 200)
          response.is_from_cache = True
          return response

      return None

    except Exception, e:
      return util.wrap_exception(request, e)


class ApiResponseMiddleware:
  def process_exception(self, request, exception):
    """Wraps exceptions for API requests."""
    if util.is_api_request(request):
      return util.wrap_exception(request, exception)
    return None

  def process_response(self, request, response):
    if not util.is_api_request(request):
      return response

    if not isinstance(response, HttpResponse):
      data = util.prepare_data(response)
      data['meta'] = {
        'result': 'ok'
      }
      response = util.build_response(request, data, 200)

      if request.method == 'GET' and response.status_code == 200:
        if not getattr(response, 'is_from_cache', False):
          request.kbcache.gen_set(cache_key(request), data)
    response['Cache-Control'] = 'max-age=0'
    return response


def cache_key(request):
  return 'api:%s' % request.get_full_path()



"""Defines HAVE_* flags for several optional packages."""

import imp
from django.core.exceptions import ImproperlyConfigured

try:
  import debug_toolbar
  HAVE_DEBUG_TOOLBAR = True
except ImportError:
  HAVE_DEBUG_TOOLBAR = False

try:
  import raven.contrib.django
  HAVE_RAVEN = True
except ImportError:
  HAVE_RAVEN = False

try:
  import django_statsd
  HAVE_STATSD = True
except ImproperlyConfigured:
  # Good enough; statsd is fussy on import.
  HAVE_STATSD = True
except ImportError:
  HAVE_STATSD = False

try:
  import storages
  HAVE_STORAGES = True
except ImportError:
  HAVE_STORAGES = False

try:
  import memcache
  HAVE_MEMCACHE = True
except ImportError:
  HAVE_MEMCACHE = False

try:
  import pylibmc
  HAVE_PYLIBMC = True
except ImportError:
  HAVE_PYLIBMC = False

try:
  import debug_toolbar_memcache
  HAVE_MEMCACHE_TOOLBAR = True
except ImportError:
  HAVE_MEMCACHE_TOOLBAR = False

try:
  import johnny
  HAVE_JOHNNY_CACHE = True
except ImportError:
  HAVE_JOHNNY_CACHE = False


# Copyright 2013 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

from django.conf import settings
from django.core.cache import cache as django_cache

import time

"""Kegbot system cache."""

# Separator
SEP = ':'

class KegbotCache:
    """Wrapper around django cache, supporting Kegbot-specific features.

    Primarily, this wrapper facilitates "generational" or "namespaced" caching,
    using two stages of cache keys: the "generation" key, and individual cache
    keys derived on it (which fall out of the cache as the generation is
    changed).
    """

    def __init__(self, prefix=None, cache=django_cache,
            generation_fn=lambda: int(time.time()),
            generation_key_name='drink_generation'):
        """Constructor.

        Args:
            prefix: If supplied, an additional prefix which will be added to all
                cache keys.  (The global KEGBOT_CACHE_PREFIX is always added).
            cache: Something implementing the django.cache interface (which is
                the default).
            generation_fn: A function which supplies the default value for
                the generation counter.  The function should return an integer.
            generation_key_name: the key name for the "generation", aka
                the namespace.
        """
        global_prefix = getattr(settings, 'KEGBOT_CACHE_PREFIX', 'kb')
        if not prefix:
            self.prefix = global_prefix
        else:
            self.prefix = SEP.join((global_prefix, prefix))
        self.cache = cache
        self.generation_key = SEP.join((self.prefix, generation_key_name))
        self.generation_fn = generation_fn

    def keyname(self, *keyparts):
        """Gets the kegbot-prefixed key name for the given base name."""
        return SEP.join((self.prefix,) + keyparts)

    def get(self, basename, default=None):
        """Wrapper around `self.cache.get()`."""
        return self.cache.get(self.keyname(basename), default)

    def set(self, basename, value, timeout=None):
        """Wrapper around `self.cache.set()`."""
        self.cache.set(self.keyname(basename), value, timeout)

    def add(self, basename, value, timeout=None):
        """Wrapper around `self.cache.add()`."""
        return self.cache.add(self.keyname(basename), value, timeout)

    def incr(self, basename, delta=1):
        """Wrapper around `self.cache.incr()`."""
        return self.cache.incr(self.keyname(basename), delta)

    def decr(self, basename, delta=1):
        """Wrapper around `self.cache.decr()`."""
        return self.cache.decr(self.keyname(basename), delta)

    ### Generational functions.

    def get_generation(self):
        """Returns the value of the current generation.

        Based on:
          https://code.google.com/p/memcached/wiki/NewProgrammingTricks
        """
        generation = self.cache.get(self.generation_key)
        if not generation:
            generation = self.generation_fn()
            if not self.cache.add(self.generation_key, generation):
                # Lost the race.
                generation = self.cache.get(self.generation_key)
        if generation is None:
            raise ValueError('Cache backend returned None')
        return generation

    def update_generation(self):
        """Increments the current generation."""
        self.get_generation()
        try:
            self.cache.incr(self.generation_key, 1)
        except ValueError:
            # Increment failed! Generation must not exist.
            self.cache.add(self.generation_key, self.generation_fn())

    def gen_keyname(self, *keyparts):
        """Like `keyname()`, but returns a key namespaced by the generation."""
        return self.keyname(*(keyparts + (str(self.get_generation()),)))

    def gen_get(self, basename, default=None):
        """Like `get()`, but uses a key namespaced by the generation."""
        return self.get(self.gen_keyname(basename), default)

    def gen_set(self, basename, value, timeout=None):
        """Like `set()`, but uses a key namespaced by the generation."""
        self.set(self.gen_keyname(basename), value, timeout)

    def gen_add(self, basename, value, timeout=None):
        """Like `add()`, but returns a key namespaced by the generation."""
        return self.cache.add(self.gen_keyname(basename), value, timeout)

    def gen_incr(self, basename, delta=1):
        """Like `incr()`, but returns a key namespaced by the generation."""
        return self.cache.incr(self.gen_keyname(basename), delta)

    def gen_decr(self, basename, delta=1):
        """Like `decr()`, but returns a key namespaced by the generation."""
        return self.cache.decr(self.gen_keyname(basename), delta)





from django import forms
from django.contrib.auth.forms import AuthenticationForm

from registration.models import RegistrationProfile
from registration.forms import RegistrationForm

from pykeg.core import models

class LoginForm(AuthenticationForm):
  next_page = forms.CharField(required=False, widget=forms.HiddenInput)


class KegbotRegistrationForm(RegistrationForm):
  def save(self, profile_callback=None):
    new_user = RegistrationProfile.objects.create_inactive_user(username=self.cleaned_data['username'],
                                                                password=self.cleaned_data['password1'],
                                                                email=self.cleaned_data['email'],
                                                                send_email=False,
                                                                profile_callback=profile_callback)
    new_user.is_active = True
    new_user.save()
    return new_user


class MugshotForm(forms.Form):
  new_mugshot = forms.ImageField(required=True)


class RegenerateApiKeyForm(forms.Form):
  pass




# Copyright 2010 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

from django.core.management.base import CommandError
from django.core.management.base import NoArgsCommand
from django.db import transaction

from pykeg.core import models
from pykeg.core import stats
from pykeg.core.management.commands.common import progbar

class Command(NoArgsCommand):
  help = u'Regenerate all cached stats.'
  args = '<none>'

  @transaction.commit_on_success()
  def handle(self, **options):
    stats.invalidate(None)

    drinks = models.Drink.objects.all().order_by('id')
    num_drinks = len(drinks)
    pos = 0
    for d in drinks:
      pos += 1
      progbar('regenerating stats', pos, num_drinks)
      stats.generate(d, invalidate_first=False)

    print ''
    print 'done!'



#!/usr/bin/env python

"""Kegbot Beer Kegerator Server package.

Kegbot is a hardware and software system to record and monitor access to a beer
kegerator.  For more information and documentation, see http://kegbot.org/
"""

from setuptools import setup, find_packages
from pykeg import __version__ as VERSION

DOCLINES = __doc__.split('\n')

SHORT_DESCRIPTION = DOCLINES[0]
LONG_DESCRIPTION = '\n'.join(DOCLINES[2:])
DEPENDENCIES = [
  'kegbot-pyutils == 0.1.7',
  'kegbot-api == 0.1.8',

  'Django == 1.5.1',
  'django-imagekit == 2.0.2',
  'django-registration == 0.8',
  'django-socialregistration == 0.5.10',
  'django-bootstrap-pagination == 0.1.10',

  'Celery == 3.0.19',
  'django-celery == 3.0.17',
  'django-kombu == 0.9.4',

  'South == 0.8.1',
  'django-crispy-forms == 1.2.8',
  'django-icanhaz == 0.2.1',
  'django_nose == 1.1',
  'facebook-sdk == 0.4.0',
  'foursquare == 20130402',
  'gunicorn == 0.17.4',
  'MySQL-python == 1.2.4',
  'pillow == 1.7.8',
  'protobuf == 2.5.0',
  'python-gflags == 2.0',
  'python-memcached == 1.51',
  'pytz == 2013b',
  'tweepy == 2.0',
  'mock == 1.0.1',
]

def setup_package():
  setup(
      name = 'kegbot',
      version = VERSION,
      description = SHORT_DESCRIPTION,
      long_description = LONG_DESCRIPTION,
      author = 'mike wakerly',
      author_email = 'opensource@hoho.com',
      url = 'http://kegbot.org/',
      packages = find_packages(),
      scripts = [
        'bin/kegbot',
        'bin/setup-kegbot.py',
      ],
      install_requires = DEPENDENCIES,
      dependency_links = [
          'https://github.com/rem/python-protobuf/tarball/master#egg=protobuf-2.4.1',
      ],
      include_package_data = True,
      entry_points = {
        'console_scripts': ['instance=django.core.management:execute_manager'],
      },
  )

if __name__ == '__main__':
  setup_package()



# Copyright 2013 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

import datetime
from django.utils import timezone
from django.utils.importlib import import_module
from django.core.exceptions import ImproperlyConfigured
from django.conf.urls import patterns
from django.conf.urls import url
from django.conf import settings

from . import plugin

MAX_TASK_AGE = datetime.timedelta(hours=1)

_CACHED_PLUGINS = None

def get_plugin_class(name):
    try:
        module_path, member_name = name.rsplit(".", 1)
        module = import_module(module_path)
        cls = getattr(module, member_name)
    except (ValueError, ImportError, AttributeError), e:
        raise ImproperlyConfigured("Could not import plugin %s: %s" % (name, e))

    if not issubclass(cls, plugin.Plugin):
        raise ImproperlyConfigured('%s does not subclass plugin.Plugin', name)

    return cls

def get_plugins():
    global _CACHED_PLUGINS
    if _CACHED_PLUGINS is None:
        plugin_names = getattr(settings, 'KEGBOT_PLUGINS', [])
        plugins = []
        for name in plugin_names:
            cls = get_plugin_class(name)
            datastore = plugin.PluginDatastore(cls.get_short_name())
            plugin_obj = cls(datastore)
            plugins.append(plugin_obj)
        _CACHED_PLUGINS = plugins
    return _CACHED_PLUGINS

def get_admin_urls():
    urls = []
    for plugin in get_plugins():
        urls += _to_urls(plugin.get_extra_admin_views(), plugin.get_short_name())
    return patterns('', *urls)

def get_account_urls():
    urls = []
    for plugin in get_plugins():
        urls += _to_urls(plugin.get_extra_user_views(), plugin.get_short_name())
    return patterns('', *urls)

def _to_urls(urllist, short_name):
    urls = []
    for regex, fn, viewname in urllist:
        regex = 'plugin/%s/%s' % (short_name, regex)
        viewname = 'plugin-%s-%s' % (short_name, viewname)
        urls.append(url(regex, fn, name=viewname))
    return urls

def is_stale(time, now=None):
  if not now:
    now = timezone.now()
  return (time + MAX_TASK_AGE) <= now

def get_logger(name):
  try:
    from celery.utils.log import get_task_logger
    logger = get_task_logger(name)
  except ImportError:
    import logging
    logger = logging.getLogger(name)
  return logger



from django.conf.urls import patterns
from django.conf.urls import url

from . import views

urlpatterns = patterns('pykeg.web.kegweb.views',
      ### main page
      url(r'^$', 'index', name='kb-home'),

      ### stats
      url(r'^stats/$', 'system_stats', name='kb-stats'),

      ### kegs
      url(r'^kegs/$', views.KegListView.as_view(), name='kb-kegs'),
      url(r'^kegs/(?P<keg_id>\d+)', 'keg_detail', name='kb-keg'),

      ### drinkers
      url(r'^drinkers/(?P<username>[\w@.+-_]+)/$', 'user_detail', name='kb-drinker'),

      ### drinks
      url(r'^drinks/(?P<drink_id>\d+)', 'drink_detail', name='kb-drink'),
      url(r'^drink/(?P<drink_id>\d+)', 'short_drink_detail'),
      url(r'^d/(?P<drink_id>\d+)', 'short_drink_detail', name='kb-drink-short'),

      ### sessions
      url(r'^session/(?P<session_id>\d+)/$', 'short_session_detail'),
      url(r'^s/(?P<session_id>\d+)/$', 'short_session_detail', name='kb-session-short'),

      url(r'^sessions/$', views.SessionArchiveIndexView.as_view(), name='kb-sessions'),
      url(r'^sessions/(?P<year>\d{4})/$', views.SessionYearArchiveView.as_view(), name='kb-sessions-year'),
      url(r'^sessions/(?P<year>\d{4})/(?P<month>\d+)/$',
          views.SessionMonthArchiveView.as_view(month_format='%m'),
          name='kb-sessions-month'),
      url(r'^sessions/(?P<year>\d{4})/(?P<month>\d+)/(?P<day>\d+)/$',
          views.SessionDayArchiveView.as_view(month_format='%m'),
          name='kb-sessions-day'),
      url(r'^sessions/(?P<year>\d+)/(?P<month>\d+)/(?P<day>\d+)/(?P<pk>\d+)/$',
          views.SessionDateDetailView.as_view(month_format='%m'),
          name='kb-session-detail'),

)




from django.conf import settings
from django.conf.urls import patterns
from django.conf.urls import url

urlpatterns = patterns('pykeg.web.setup_wizard.views',
  url(r'^$', 'start', name='setup_wizard_start'),
  url(r'^create/$', 'create_or_import', name='setup_create_or_import'),
  url(r'^settings/$', 'site_settings', name='setup_site_settings'),
  url(r'^admin-user/$', 'admin', name='setup_admin'),
  url(r'^finished/$', 'finish', name='setup_finish'),
)



# Copyright 2010 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

"""Methods to generate cached statistics from drinks."""

import copy
import inspect
import itertools
import logging

from django.db import transaction

from pykeg.core import models
from kegbot.util import util

STAT_MAP = {}

def stat(statname):
  def decorate(f):
    setattr(f, 'statname', statname)
    return f
  return decorate

class StatsBuilder:
  def __init__(self, drink, drink_qs, previous=None):
    self.drink = drink
    self.drinks = drink_qs
    self.previous = util.AttrDict(copy.deepcopy(previous))
    self.STAT_MAP = {}
    for name, fn in inspect.getmembers(self, inspect.ismethod):
      if hasattr(fn, 'statname'):
        self.STAT_MAP[fn.statname] = fn

  def build(self, tag=None):
    stats = util.AttrDict()
    if not self.drink:
      return stats
    if self.previous:
      stats = copy.deepcopy(self.previous)
    for statname, fn in self.STAT_MAP.iteritems():
      previous = self.previous.get(statname, None)
      val = fn(previous)
      if val is None:
        raise ValueError('Stat generator for %s returned None' % statname)
      stats[statname] = val
    return stats

  @stat('last_drink_id')
  def LastDrinkId(self, previous):
    return self.drink.id

  @stat('total_volume_ml')
  def TotalVolume(self, previous):
    if not previous:
      return sum(drink.volume_ml for drink in self.drinks)
    else:
      return previous + self.drink.volume_ml

  @stat('total_pours')
  def TotalPours(self, previous):
    if not previous:
      return self.drinks.count()
    else:
      return previous + 1

  @stat('average_volume_ml')
  def AverageVolume(self, previous):
    if not previous:
      count = self.drinks.count()
      average = 0.0
      if count:
        average = sum(drink.volume_ml for drink in self.drinks) / float(count)
      return average
    else:
      vol = self.previous.total_volume_ml
      count = self.previous.total_pours
      vol += self.drink.volume_ml
      count += 1
      return vol / float(count)

  @stat('greatest_volume_ml')
  def GreatestVolume(self, previous):
    if not previous:
      res = 0
      drinks = self.drinks.order_by('-volume_ml')
      if drinks.count():
        res = drinks[0].volume_ml
      return float(res)
    else:
      return float(max(self.drink.volume_ml, previous))

  @stat('greatest_volume_id')
  def GreatestVolumeId(self, previous):
    if not previous:
      res = 0
      drinks = self.drinks.order_by('-volume_ml')
      if drinks.count():
        res = drinks[0].id
      return res
    else:
      if self.drink.volume_ml > self.previous.greatest_volume_ml:
        return self.drink.id
      return previous

  @stat('volume_by_day_of_week')
  def VolumeByDayOfweek(self, previous):
    if not previous:
      # Note: uses the session's start_time, rather than the drink's. This
      # causes late-night sessions to be reported for the day on which they were
      # started.
      volmap = {}
      for drink in self.drinks:
        weekday = str(drink.session.start_time.strftime('%w'))
        if weekday not in volmap:
          volmap[weekday] = 0.0
        volmap[weekday] += drink.volume_ml
      return volmap
    else:
      drink_weekday = str(self.drink.session.start_time.strftime('%w'))
      previous[drink_weekday] = previous.get(drink_weekday, 0) + self.drink.volume_ml
      return previous

  @stat('registered_drinkers')
  def RegisteredDrinkers(self, previous):
    if previous is None:
      drinkers = set()
      for drink in self.drinks:
        if drink.user:
          drinkers.add(str(drink.user.username))
      return list(drinkers)
    else:
      if self.drink.user:
        username = str(self.drink.user.username)
        if username not in previous:
          previous.append(str(username))
      return previous

  @stat('sessions_count')
  def SessionsCount(self, previous):
    if not previous:
      all_sessions = set()
      for drink in self.drinks:
        all_sessions.add(drink.session.id)
      return len(all_sessions)
    else:
      first_drink = self.drink.session.drinks.order_by('id')[0]
      if self.drink.id == first_drink.id:
        previous += 1
      return previous

  @stat('volume_by_year')
  def VolumeByYear(self, previous):
    if not previous:
      ret = {}
      for drink in self.drinks:
        year = str(drink.time.year)
        ret[year] = ret.get(year, 0) + drink.volume_ml
      return ret
    else:
      year = str(self.drink.time.year)
      previous[year] = previous.get(year, 0) + self.drink.volume_ml
      return previous

  @stat('has_guest_pour')
  def HasGuestPour(self, previous):
    if not previous:
      return self.drinks.filter(user_id=None).count() > 0
    else:
      return bool(previous or self.drink.user is None)

  @stat('volume_by_drinker')
  def VolumeByDrinker(self, previous):
    if not previous:
      volmap = {}
      for drink in self.drinks:
        if drink.user:
          u = str(drink.user.username)
        else:
          u = ''
        volmap[u] = float(volmap.get(u, 0) + drink.volume_ml)
      return volmap
    else:
      if self.drink.user:
        u = str(self.drink.user.username)
      else:
        u = ''
      orig = previous.get(u, 0)
      previous[u] = float(orig + self.drink.volume_ml)
      return previous


def invalidate(drink):
  """Clears all statistics.

  Args:
      drink: The starting point to invalidate; all stats starting from this
          drink will be deleted.  If None, deletes *all* stats.
  """
  with transaction.commit_on_success():
    if drink:
      models.SystemStats.objects.filter(drink_id__gte=drink.id).delete()
      models.KegStats.objects.filter(drink_id__gte=drink.id).delete()
      models.UserStats.objects.filter(drink_id__gte=drink.id).delete()
      models.SessionStats.objects.filter(drink_id__gte=drink.id).delete()
    else:
      models.SystemStats.objects.all().delete()
      models.KegStats.objects.all().delete()
      models.UserStats.objects.all().delete()
      models.SessionStats.objects.all().delete()

def _get_previous(drink, qs):
  qs = qs.filter(drink_id__lt=drink.id).order_by('-id')[:1]
  if qs.count():
    return qs[0].stats
  return {}

def build_system_stats(drink):
  """Builds (but does not save) system stats dictionary for drink."""
  qs = models.Drink.objects.filter(id__lte=drink.id)
  previous = _get_previous(drink, models.SystemStats.objects.all())
  builder = StatsBuilder(drink, qs, previous)
  return builder.build('system')

def generate_system_stats(drink):
  """Builds and saves system stats record for drink."""
  stats = build_system_stats(drink)
  return models.SystemStats.objects.create(drink=drink, stats=stats)

def build_keg_stats(drink):
  """Builds (but does not save) keg stats dictionary for drink."""
  if drink.keg:
    qs = models.Drink.objects.filter(id__lte=drink.id, keg_id=drink.keg.id)
    previous = _get_previous(drink,
        models.KegStats.objects.filter(keg=drink.keg))
    builder = StatsBuilder(drink, qs, previous)
    return builder.build('keg ' + str(drink.keg.id))

def generate_keg_stats(drink):
  """Builds and saves keg stats record for drink."""
  stats = build_keg_stats(drink)
  if stats:
    return models.KegStats.objects.create(drink=drink, keg=drink.keg, stats=stats)

def build_user_stats(drink):
  """Builds (but does not save) user stats dictionary for drink."""
  if drink.user:
    user_id = drink.user.id
  else:
    user_id = None
  qs = models.Drink.objects.filter(id__lte=drink.id, user_id=user_id)
  previous = _get_previous(drink,
      models.UserStats.objects.filter(user=drink.user))
  builder = StatsBuilder(drink, qs, previous)
  return builder.build('user ' + str(user_id))

def generate_user_stats(drink):
  """Builds and saves user stats record for drink."""
  stats = build_user_stats(drink)
  return models.UserStats.objects.create(drink=drink, user=drink.user, stats=stats)

def build_session_stats(drink):
  """Builds (but does not save) session stats dictionary for drink."""
  if drink.session:
    qs = models.Drink.objects.filter(id__lte=drink.id, session_id=drink.session.id)
    previous = _get_previous(drink,
        models.SessionStats.objects.filter(session=drink.session))
    builder = StatsBuilder(drink, qs, previous)
    return builder.build('session ' + str(drink.session.id))

def generate_session_stats(drink):
  """Builds and saves session stats record for drink."""
  stats = build_session_stats(drink)
  if stats:
    return models.SessionStats.objects.create(drink=drink,
        session=drink.session, stats=stats)

def generate(drink, invalidate_first=True):
  """Generate all stats for this drink.

  Args:
      drink: The drink.
      invalidate_first: If True, deletes any existing records for (or after)
          this drink first.
  """
  with transaction.commit_on_success():
    if invalidate_first:
      invalidate(drink)

    generate_system_stats(drink)
    generate_keg_stats(drink)
    generate_user_stats(drink)
    generate_session_stats(drink)

if __name__ == '__main__':
  import cProfile
  command = """main()"""
  cProfile.runctx( command, globals(), locals(), filename="stats.profile" )




#!/usr/bin/env python
#
# Copyright 2010 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

"""Kegweb main views."""

from django.http import Http404
from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.core.urlresolvers import reverse
from django.shortcuts import render_to_response
from django.shortcuts import redirect
from django.template import RequestContext
from django.views.decorators.http import require_POST
from django.contrib.auth.views import password_change as password_change_orig
from django.contrib.auth.views import password_change_done as password_change_done_orig

from pykeg.core import models
from pykeg.web.kegweb import forms

@login_required
def account_main(request):
  context = RequestContext(request)
  context['user'] = request.user
  return render_to_response('account/index.html', context_instance=context)

@login_required
def edit_mugshot(request):
  context = RequestContext(request)
  context['mugshot_form'] = forms.MugshotForm()
  if request.method == 'POST':
    form = forms.MugshotForm(request.POST, request.FILES)
    context['mugshot_form'] = form
    if form.is_valid():
      pic = models.Picture.objects.create()
      image = request.FILES['new_mugshot']
      pic.image.save(image.name, image)
      pic.save()

      profile = request.user.get_profile()
      profile.mugshot = pic
      profile.save()
  return render_to_response('account/mugshot.html', context_instance=context)

@login_required
@require_POST
def regenerate_api_key(request):
  form = forms.RegenerateApiKeyForm(request.POST)
  if form.is_valid():
    key, is_new = models.ApiKey.objects.get_or_create(user=request.user)
    key.regenerate()
    key.save()
  return redirect('kb-account-main')

def password_change(request, *args, **kwargs):
  kwargs['template_name'] = 'account/password_change.html'
  kwargs['post_change_redirect'] = reverse('password_change_done')
  return password_change_orig(request, *args, **kwargs)

def password_change_done(request):
  return password_change_done_orig(request, 'account/password_change_done.html')

@login_required
def plugin_settings(request, plugin_name):
  context = RequestContext(request)
  plugin = request.plugins.get(plugin_name, None)
  if not plugin:
    raise Http404('Plugin "%s" not loaded' % plugin_name)

  view = plugin.get_user_settings_view()
  if not view:
    raise Http404('No user settings for this plugin')

  return view(request, plugin)



# Copyright 2013 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

from django.core.cache import cache as django_cache
from django.test import TransactionTestCase
from django.test.utils import override_settings

from .cache import KegbotCache

class KegbotCacheTest(TransactionTestCase):

    def setUp(self):
        django_cache.clear()

    def test_basics(self):
        cache = KegbotCache(generation_fn=lambda: 100)

        # keyname, get, set
        self.assertEquals('kb:foo', cache.keyname('foo'))
        self.assertEquals(None, django_cache.get('kb:foo'))
        self.assertEquals(123, django_cache.get('kb:foo', 123))
        cache.set('foo', 'bar')
        self.assertEquals('bar', django_cache.get('kb:foo'))

        # generation
        self.assertEquals(None, django_cache.get(cache.generation_key))
        gen = cache.get_generation()
        self.assertEquals(100, gen)
        self.assertEquals('kb:foo:100', cache.gen_keyname('foo'))
        self.assertEquals(100, django_cache.get(cache.generation_key))

        cache.gen_set('test', 555)
        self.assertEquals(555, cache.gen_get('test'))
        cache.update_generation()
        self.assertEquals(101, cache.get_generation())
        self.assertEquals(None, cache.gen_get('test'))

    @override_settings(KEGBOT_CACHE_PREFIX='other')
    def test_prefix_other(self):
        """Verifies settings.KEGBOT_CACHE_PREFIX is respected."""
        cache = KegbotCache(generation_fn=lambda: 100)
        self.assertEquals('other:foo', cache.keyname('foo'))
        self.assertEquals('other:foo:100', cache.gen_keyname('foo'))

    def test_generation_update(self):
        """Tests updating the generation succeeds even when missing."""
        cache = KegbotCache(generation_fn=lambda: 100)
        cache.update_generation()
